(********************************************************************
 * COPYRIGHT -- Danfoss
 ********************************************************************
 * Library: DDS_BasCam
 * File: DDS_BasCam.typ
 * Author: desc0724
 * Created: September 17, 2014
 ********************************************************************
 * Data types of library DDS_BasCam
 ********************************************************************)

TYPE
	DD_FACTOR_DDS :
	STRUCT
		(*Factor: Numerator / Denominator*)
		Numerator :		DINT := 0;
		Denominator : 	DINT := 0;
	END_STRUCT;
	DD_CONTROL_PARAMETER_DDS :
	STRUCT
		(*Contains parameters for setting the speed and position controller*)
		SpeedP : 	REAL := 0.0; 	(*Proportional part of the speed controller*)
		SpeedI : 		REAL := 0.0; 	(*Integral part of the speed controller*)
		SpeedD : 	REAL := 0.0; 	(*Differential part of the speed controller*)
		Inertia : 		REAL := 0.0; 	(*Inertia that is used for feed-forward calculations. Given in kg*m*)
		PositionP : 	REAL := 0.0; 	(*Proportional part of the position controller*)
		PositionD : 	REAL := 0.0; 	(*Differential part of the position controller*)
	END_STRUCT;
	DD_FOLLOWING_ERROR_DDS :
	STRUCT
		WindowRev : 		REAL := 0.0; 	(*Following error window. Given in revolutions.*)
		TimeOut : 		UINT := 0; 	(*Following error time out. Given in ms.*)
	END_STRUCT;
	DD_DATA_POINT_DDS :
	STRUCT
		(*Data point definition for Basic CAM*)
		MasterPosition : 	REAL := 0.0; 	(*Master Position for this data point. Given in revolutions of guide value.
									Value range: between 0 and 1 (1 not included)*)
		SlavePosition :	 REAL := 0.0; 	(*Axis position for this data point. Given in revolutions fo rotor position.
									This is the position at gear in (=motor side)*)
		Velocity : 		REAL := 0.0; 	(*Velocity of the axis in this data point. The velocity has to be given as a factor
									between the velocity of the axis in relation to the velocity of the guide value. *)
		Acceleration : 	REAL := 0.0; 	(*Acceleration of the axis in this data point. The acceleration has to be given as
									a factor between the acceleration of the axis in relation to the velocity of the guide value.*)
	END_STRUCT;
	DD_BASIC_CAM_DDS :
	STRUCT
		MasterScaling : 			DD_FACTOR_DDS; 												(*Parameters for master scaling*)
		SlaveScaling : 			DD_FACTOR_DDS; 												(*Parameters for slave scaling*)
		ControlParameterSet1 : 	DD_CONTROL_PARAMETER_DDS; 								(*Parameters for control parameter set 1*)
		ControlParameterSet2 :		DD_CONTROL_PARAMETER_DDS; 								(*Parameters for control parameter set 2*)
		FollowingError : 			DD_FOLLOWING_ERROR_DDS; 									(*Parameters for following error set-up*)
		DataPoints : 				ARRAY[1..DD_MAX_DATA_POINTS_DDS]OF DD_DATA_POINT_DDS; 	(*Array of Data Points for the Basic CAM*)
		NumberOfDataPoints : 		UINT := 0; 														(*Gives the number of used Data Points inside the array*)
	END_STRUCT;
END_TYPE
