(********************************************************************
 * COPYRIGHT -- Danfoss A/S
 ********************************************************************
 * Library: DDS_Drive
 * File: DDS_Drive.typ
 * Author: DESC0724
 * Created: Febuary 03, 2014
 ********************************************************************
 * Data types of library DDS_Drive
 *********************************************************************)

TYPE
	DD_ERROR_DDS :
	STRUCT	(*Structure for error information*)
		ErrorID : 			WORD;		(*Error identification*)
		InstanceID : 		UDINT; 		(*Identifier of FB instance that caused the error*)
		NodeID : 		UINT; 		(*Node number of the device*)
	END_STRUCT;
	DD_HOME_MODE_DDS :
	( (*Defines the method used for homing.
	Depending on this selection, the appropriate input variables are used.*)
		ddAbsolute_DDS := -3, 			(*Homing method: Homing on actual position*)
		ddPosBlock_DDS := -2, 		(*Homing method: Homing on positive block*)
		ddNegBlock_DDS := -1, 		(*Homing method: Homing on negative block*)
		ddNegLimSwitch_DDS := 17, 	(*Homing method: Homing on negative limit switch*)
		ddPosLimSwitch_DDS := 18, 	(*Homing method: Homing on positive limit switch*)
		ddPosHomeSwitch_DDS := 19, 	(*Homing method: Homing on positive home switch*)
		ddNegHomeSwitch_DDS := 21, 	(*Homing method: Homing on negative home switch*)
		ddDirect_DDS := 37 			(*Homing method: Homing on current position*)
	);
	GUIDE_VALUE_REF_DDS :
	STRUCT	(*Structure for holding the guide value*)
		PositionGuideValue : 	UDINT; 	(*Value of the position guide value. 1 cycle is scaled from 0 to 16#FFFF FFFF*)
		VelocityGuideValue : 	REAL; 	(*Velocity of the guide value. Given in rps*)
	END_STRUCT;
	MC_BUFFER_MODE_DDS :
	(	(*Defines the chronological sequence of the FBs*)
		mcAborting_DDS := 0, 	(*Actual positioning process will be aborted and replaced with new one*)
		mcBuffered_DDS := 1 	(*Actual positioning process will be continued and the next will follow*)
	);
	MC_DIRECTION_DDS :
	(	(*Defines the direction used to reach the target of a periodic axis*)
		mcCurrentDirection_DDS, 		(*Movement in current direction of active move*)
		mcNegativeDirection_DDS, 	(*Movement only in negative direction*)
		mcPositiveDirection_DDS, 		(*Movement only in positive direction*)
		mcShortestWay_DDS, 			(*Move the shortest distance to the target, either positive or negative*)
		mcLinearAxis_DDS			(*Normal movement similar to a linear axis*)
	);
	DD_EDGE_MODE_DDS :
	( (*Enumeration to specifiy digital input events.*)
		ddPositiveEdge_DDS, 			(*Only positive edges on the digital input are used as events.*)
		ddNegativeEdge_DDS,		(*Only negative edges on the digital input are used as events.*)
		ddBothEdges_DDS			(*Positive and negative edges on the digital input are used as events.*)
		);
	MC_SYNC_MODE_DDS :
	(	(*Defines the way to synchronize to a master axis*)
		mcShortest_DDS, 		(*Synchronization is done to have the shortest way for the slave*)
		mcCatchUp_DDS, 	(*Synchronization is done to catch up to the master axis (synchronization to current master cycle)*)
		mcSlowDown_DDS 	(*Synchronization is done to slow down to the master axis (synchronization one master cycle later)*)
	);
	DD_MAX_STRING_DDS :
		STRING[iDD_MAX_STRING_LENGTH];
	DD_SAMPLING_RATE_DDS :
	(
		ddFastTask_DDS := 1,
		ddSlowTask_DDS := 2,
		ddRealTimeTask_DDS := 3
	);
	DD_CAM_DELAY_MODE_DDS :
	( (*Enumeration to specifiy the activation behaviour of a CAM (with master relative).*)
		ddNoDelay_DDS := 0, 			(*Normal CAM activation behaviour*)
		ddInput1_DDS,				(*Delayed activation of CAM: Processing of CAM is delayed until digital input 1 is on.*)
		ddInput2_DDS,				(*Delayed activation of CAM: Processing of CAM is delayed until digital input 2 is on.*)
		ddInput1or2_DDS, 			(*Delayed activation of CAM: Processing of CAM is delayed until digital input 1 or 2 is on.*)
		ddInput1and2_DDS 			(*Delayed activation of CAM: Processing of CAM is delayed until digital inputs 1 and 2 are on.*)
		);
	DD_STOPPING_MODE_DDS :
	( (*Enumeration to specifiy the way of stopping the drive for this master cycle (for the rotations top command)*)
		ddCoasting_DDS := 0, 		(*Normal CAM activation behaviour*)
		ddRamp_DDS := 1, 		(*Slow down on specified ramp and stay in Operation enabled. Deceleration value needs to be specified for this mode.*)
		ddCurrentLimit_DDS := 2 	(*Slow down on current limit and stay in Operation enabled.*)
		);
	DD_CONTROL_MODE_DDS :
	(
		dd_cm_Local_DDS := 0,				(* Control mode: Local Mode *)
		dd_cm_DigitalInput_DDS := 1,			(* Control mode: Digital Input *)
		dd_cm_Automatic_DDS := 2,			(* Control mode: Automatic *)
		dd_cm_FieldBus_DDS := 3				(* Control mode: FieldBus *)
	);
	DD_DRIVE_TYPE_DDS :
	((*Enumeration to specifiy the drive type, which will be used*)
		dd_ISD_DDS := 0,		(*Integrated servo drive system for machine mounting*)
		dd_DSD_DDS,		(*Decentral servo drive system for machine mounting*)
		dd_SDM_DDS		(*Single axis servo drive*)
	);
END_TYPE
