
FUNCTION iDD_CommunicationReady_DDS : BOOL (*Checks if there is communication to the drive*)
	VAR_IN_OUT
		Axis : AXIS_REF_DDS;
	END_VAR
END_FUNCTION

FUNCTION iDD_CommunicationReady_SAB : BOOL (*Checks if there is communication to the SAB*)
	VAR_IN_OUT
		Sab : SAB_REF;
	END_VAR
END_FUNCTION

FUNCTION_BLOCK iDD_CycleInfo_DDS (*Special functionality of AS that returns the cpu time of the last read-in of the IOs*)
	VAR_OUTPUT
		NewCycle : BOOL; (*True if it is a new IO cycle*)
	END_VAR
	VAR
		time : DINT; (*Cpu time at this function call *)
		time_last : DINT; (*Cpu time at previouse call of function*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_FileOpen_DDS (*Wrapper to handle a file open command*)
	VAR_INPUT
		Execute : BOOL; (*Execute*)
		pDevice : UDINT; (*Pointer to the device name on which the file is located*)
		TargetFile : STRING[80]; (*Path of file and target file name on the drive like "DSP/fs/test.bin"*)
		Mode : iDD_FileOpenMode_DDS; (*Access mode of the file*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*The file is opened correctly*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorId : WORD; (*Error information*)
	END_VAR
	VAR_IN_OUT
		FileHandle : iDD_INTERN_FILE_HANDLE_DDS;
	END_VAR
	VAR
		IntpDevice : UDINT; (*Pointer to the device name on which the file is located*)
		IntTargetFile : STRING[80]; (*Path of file and target file name on the drive like "DSP/fs/test.bin"*)
		IntMode : iDD_FileOpenMode_DDS; (*Access mode of the file*)
		ExecuteTrigR_0 : R_TRIG; (*Rising edge of Execute*)
		FBStatus : iDD_INTERN_STATUS_DDS; (*Status value of state machine of FB*)
		IntStatus : USINT; (*internal status machine *)
		FileOpen_0 : FileOpen; (*FB to open source file*)
		FileCreate_0 : FileCreate; (*Create FB if it does not exist*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_FileRead_DDS (*Wrapper to handle a file read command*)
	VAR_INPUT
		Execute : BOOL; (*Execute*)
		pBuffer : UDINT;
		BufferLength : UDINT;
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*The string has been written*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorId : WORD; (*Error information*)
		ReadBytes : UDINT;
		EOF : BOOL;
	END_VAR
	VAR_IN_OUT
		FileHandle : iDD_INTERN_FILE_HANDLE_DDS;
	END_VAR
	VAR
		ExecuteTrigR_0 : R_TRIG; (*Rising edge of Execute*)
		FBStatus : iDD_INTERN_STATUS_DDS; (*Status value of state machine of FB*)
		FileRead_0 : FileReadEx; (*FB to read from the file*)
		IntpBuffer : UDINT;
		IntLength : UDINT;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_FileWrite_DDS (*Wrapper to handle a file write command*)
	VAR_INPUT
		Execute : BOOL; (*Execute*)
		pBuffer : UDINT;
		Length : UDINT;
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*The string has been written*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorId : WORD; (*Error information*)
	END_VAR
	VAR_IN_OUT
		FileHandle : iDD_INTERN_FILE_HANDLE_DDS;
	END_VAR
	VAR
		ExecuteTrigR_0 : R_TRIG; (*Rising edge of Execute*)
		FBStatus : iDD_INTERN_STATUS_DDS; (*Status value of state machine of FB*)
		FileWrite_0 : FileWrite; (*FB to write to file*)
		IntpBuffer : UDINT;
		IntLength : UDINT;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_FileClose_DDS (*Wrapper to handle a file close command*)
	VAR_INPUT
		Execute : BOOL; (*Execute*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*The string has been written*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorId : WORD; (*Error information*)
	END_VAR
	VAR_IN_OUT
		FileHandle : iDD_INTERN_FILE_HANDLE_DDS;
	END_VAR
	VAR
		ExecuteTrigR_0 : R_TRIG; (*Rising edge of Execute*)
		FBStatus : iDD_INTERN_STATUS_DDS; (*Status value of state machine of FB*)
		FileClose_0 : FileClose; (*FB to close source file*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_FileTransfer_DDS (*File transfer over SDO*)
	VAR_INPUT
		Execute : BOOL; (*Execute*)
		SourceFileName : DD_MAX_STRING_DDS; (*Name of the file to be read from the PLC*)
		TargetFile : DD_MAX_STRING_DDS; (*Path of file and target file name on the drive like "DSP/fs/test.bin"*)
		NodeNumber : UINT; (*Node number of the device*)
		SlotName : STRING[80];
		pDevice : UDINT; (*Pointer to the name of the PLC device on which the file is located*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*File transfer process finished successfully*)
		Busy : BOOL; (*Still transfering*)
		Error : BOOL; (*True if transfering process was not successfull *)
		ErrorId : WORD; (*Error ID in case of error, does not include SDO-Abort-Error*)
	END_VAR
	VAR_IN_OUT
		FosSemaphore : iDD_Semaphore_DDS; (*Semaphore of the device for FoS transfer*)
	END_VAR
	VAR
		IntpDevice : UDINT;
		IntSourceFileName : DD_MAX_STRING_DDS; (*Reference to the name of the file to be read from the PLC*)
		IntTargetFile : DD_MAX_STRING_DDS; (*Path of file and target file name on the drive like "DSP/fs/test.bin"*)
		IntNodeNumber : UINT; (*Node number of the device*)
		IntSlotName : STRING[80];
		IntErrorId : WORD; (*To be used if there is an error inside the FB. The ErrorId output may only be assigned at the final state*)
		Buffer : ARRAY[0..iDD_MAX_BUFFER_INDEX] OF USINT; (*Buffer to hold the source file content*)
		ExecuteTrigR_0 : R_TRIG; (*Rising edge of Execute*)
		FBStatus : iDD_INTERN_STATUS_DDS; (* Status value of state machine of FB*)
		DDFileStatus : iDD_INTERN_FILETRANS_STAT_DDS; (*internal status machine *)
		SdoWrite_0 : iDD_SdoWriteUniversal_DDS; (*Internal SDO-Write-FB*)
		FileOpen_0 : iDD_FileOpen_DDS;
		FileRead_0 : iDD_FileRead_DDS;
		FileClose_0 : iDD_FileClose_DDS;
		EofReached : BOOL; (*TRUE if the End of File is reached*)
		AbortAndRestart : BOOL; (*Rising edge during Busy was detected=> Finish transfer and restart*)
		DDFileStatus_BeforeDelay : iDD_INTERN_FILETRANS_STAT_DDS; (*state of internal state machine before entering the delay state*)
		DelayTimer : iDD_TON_DDS; (*delay timer for waiting when device is not ready to accept the next segment*)
		WriteAttemptCounter : UINT; (*Counting of failed attempts to resend the same data *)
		HoldSemaphore : BOOL; (*Keeps track if this FB holds the semaphore or not*)
		FileHandle : iDD_INTERN_FILE_HANDLE_DDS;
	END_VAR
	VAR CONSTANT
		INFO_OBJ_INDEX : UINT := 16#5060; (*Index of the data object*)
		DATA_OBJ_INDEX : UINT := 16#5061; (*Index of the info object*)
	END_VAR
	VAR
		START_WRITE : USINT := 1; (*Constant for "Start writing file"*)
		FINISH_TRANSFER : USINT := 2; (*Constant for "Finish transfer"*)
		ABORT_TRANSFER : USINT := 3; (*Constant for "Abort transfer"*)
		MAX_WRITE_ATTEMPTS : UINT := 2000; (*Maximum number of write attempts to send data*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION iDD_FormatArgToStr_DDS : UDINT
	VAR_INPUT
		bSign : BOOL; (* Sign prefix flag *)
		bBlank : BOOL; (* Blank prefix flag *)
		bNull : BOOL; (* Null prefix flag *)
		bHash : BOOL; (* Hash prefix flag *)
		bLAlign : BOOL; (* FALSE => Right align (default), TRUE => Left align *)
		bWidth : BOOL; (* FALSE => no width padding, TRUE => blank or zeros padding enabled *)
		iWidth : INT; (* Width length parameter *)
		iPrecision : INT; (* Precision length parameter *)
		eFmtType : iDD_INTERN_TYPEFIELDPARA_DDS; (* Format type field parameter *)
		arg : iDD_INTERN_ARG_DDS; (* Format argument *)
	END_VAR
	VAR_IN_OUT
		sOut : DD_MAX_STRING_DDS;
	END_VAR
	VAR CONSTANT
		FORMAT_HASH_PREFIX_STRING : ARRAY[iHASHPREFIX_IEC..iHASHPREFIX_STDC,iTYPEFIELD_UNKNOWN..iTYPEFIELD_EL] OF STRING[10] := ['','2#','8#','','','','','','16#','16#','','','','','0','','','','','','0X','0x','','']; (*	UNKN,	B,	 	O,		U,		C,		F, 		D,		S, 		X, 		x, 		E, 		e	*)
		FORMAT_PREFIX_SUPPORT_MASK : ARRAY[iPREFIXFLAG_SIGN..iPREFIXFLAG_HASH,iTYPEFIELD_UNKNOWN..iTYPEFIELD_EL] OF BOOL := [FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,TRUE,FALSE,FALSE,FALSE,TRUE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,TRUE,FALSE,FALSE,FALSE,TRUE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,FALSE,TRUE,FALSE,FALSE,TRUE,TRUE,FALSE,TRUE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,TRUE,FALSE,FALSE]; (*	UNKN,		B,	 		O,			U,			C,			F, 			D,			S, 			X, 			x, 			E, 			e	*) (* plc var type support mask
		UNKN,	BYTE,	 WORD,	DWORD,	REAL,	LREAL, 	SINT,	INT, 	DINT, 	USINT, 	UINT, 	UDINT, 	STRING, BOOL,	BIGTYPE,ULARGE,	UHUGE,	LARGE,	HUGE 	*)
		FORMAT_TYPE_SUPPORT_MASK : ARRAY[iTYPEFIELD_UNKNOWN..iTYPEFIELD_EL,iARGTYPE_UNKNOWN..iARGTYPE_HUGE] OF BOOL := [FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,TRUE,TRUE,TRUE,FALSE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,TRUE,TRUE,TRUE,FALSE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,TRUE,TRUE,FALSE,FALSE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,TRUE,TRUE,FALSE,FALSE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,TRUE,TRUE,TRUE,FALSE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,TRUE,TRUE,TRUE,FALSE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE];
		ARGTYPE_IS_SIGNED_TYPE : ARRAY[iARGTYPE_UNKNOWN..iARGTYPE_HUGE] OF BOOL := [FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,TRUE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,TRUE];
		ARGTYPE_IS_FLOAT_TYPE : ARRAY[iARGTYPE_UNKNOWN..iARGTYPE_HUGE] OF BOOL := [FALSE,FALSE,FALSE,FALSE,TRUE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE];
	END_VAR
	VAR
		pOut : REFERENCE TO BYTE;
		longword : DWORD;
		double : LREAL;
		single : REAL;
		short : SINT;
		small : INT;
		longint : DINT;
		iPaddingLen : INT;
		iCurrLen : INT;
		iDD_DWORD_TO_BINSTR_DDS_0 : iDD_DWORD_TO_BINSTR_DDS;
		iDD_DINT_TO_DECSTR_DDS_0 : iDD_DINT_TO_DECSTR_DDS;
		iDD_DWORD_TO_DECSTR_DDS_0 : iDD_DWORD_TO_DECSTR_DDS;
		iDD_LREAL_TO_FMTSTR_DDS_0 : iDD_LREAL_TO_FMTSTR_DDS;
		iDD_DWORD_TO_OCTSTR_DDS_0 : iDD_DWORD_TO_OCTSTR_DDS;
		iDD_DWORD_TO_HEXSTR_DDS_0 : iDD_DWORD_TO_HEXSTR_DDS;
	END_VAR
END_FUNCTION

FUNCTION_BLOCK iDD_FormatString_DDS
	VAR_INPUT
		sFormat : DD_MAX_STRING_DDS;
		arg1 : iDD_INTERN_ARG_DDS;
		arg2 : iDD_INTERN_ARG_DDS;
		arg3 : iDD_INTERN_ARG_DDS;
		arg4 : iDD_INTERN_ARG_DDS;
		arg5 : iDD_INTERN_ARG_DDS;
		arg6 : iDD_INTERN_ARG_DDS;
		arg7 : iDD_INTERN_ARG_DDS;
		arg8 : iDD_INTERN_ARG_DDS;
		arg9 : iDD_INTERN_ARG_DDS;
		arg10 : iDD_INTERN_ARG_DDS;
	END_VAR
	VAR_OUTPUT
		bError : BOOL;
		nErrId : UDINT;
		sOut : DD_MAX_STRING_DDS;
	END_VAR
	VAR
		pFormat : REFERENCE TO BYTE; (*pointer to the format string *)
		pOut : REFERENCE TO BYTE; (* pointer to the result string *)
		iRemOutLen : INT; (* Max remaining length of sOut buffer*)
		bValid : BOOL; (* if set, the string character is valid format parameter *)
		stFmt : iDD_INTERN_FORMATPARAMS_DDS;
		nArrayElem : INT;
		nArgument : UDINT; (*parArgs		: ARRAY[1..FORMAT_MAX_ARGS] OF REFERENCE TO T_Arg;*)
		parArgs : ARRAY[1..iDD_FORMAT_MAX_ARGS] OF iDD_INTERN_ARG_DDS;
		sArgStr : DD_MAX_STRING_DDS;
	END_VAR
	VAR CONSTANT
		FORMAT_DEFAULT_PRECISION : ARRAY[iTYPEFIELD_UNKNOWN..iTYPEFIELD_EL] OF INT := [-1,4(1),6,1,-1,2(1),2(-1)]; (* plc var type support mask *) (*	UNKN,	BYTE,	 WORD,	DWORD,	REAL,	LREAL, 	SINT,	INT, 	DINT, 	USINT, 	UINT, 	UDINT, 	STRING, BOOL,	BIGTYPE,ULARGE,	UHUGE,	LARGE,	HUGE 	*) (* TYPEFIELD_UNKNOWN *)
		ARGTYPE_IS_UNSIGNED_TYPE : ARRAY[iARGTYPE_UNKNOWN..iARGTYPE_HUGE] OF BOOL := [FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,TRUE,TRUE,FALSE,FALSE,FALSE,TRUE,TRUE,FALSE,FALSE]; (*ARGTYPE_IS_STRING_TYPE : ARRAY[ARGTYPE_UNKNOWN..ARGTYPE_HUGE] OF BOOL := [FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE];*)
		ARGTYPE_IS_BIT_TYPE : ARRAY[iARGTYPE_UNKNOWN..iARGTYPE_HUGE] OF BOOL := [FALSE,TRUE,TRUE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE];
	END_VAR
END_FUNCTION_BLOCK

FUNCTION iDD_ResetErrorInfo_DDS : BOOL
	VAR_IN_OUT
		ErrorInfo : DD_ERROR_DDS;
	END_VAR
END_FUNCTION

FUNCTION_BLOCK iDD_SdoRead_DDS (*Internal FB to manage sending SDO requests*)
	VAR_INPUT
		Enable : BOOL; (*Get the value of the object continuously while enabled*)
		Index : UINT; (*Index of the object to be read*)
		Subindex : USINT; (*Subindex of the object to be read*)
		pData : UDINT; (*Reference to the data to be read to*)
		DataLen : UDINT; (*Length of the data buffer *)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Reading process finished successfully*)
		Busy : BOOL; (*Still reading*)
		Error : BOOL; (*True if reading process was not successful*)
		ErrorId : WORD; (*Error ID in case of error, dose not include SDO-Abort-Error*)
		ReadLen : UDINT; (*Number of read bytes*)
		SdoAbort : DWORD; (*SDO-Abort-Code in case of an SDO Error *)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to Axis*)
	END_VAR
	VAR
		InternalSdoRead : iDD_SdoReadUniversal_DDS;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_SdoRead_SAB (*Internal FB to manage sending SDO requests*)
	VAR_INPUT
		Enable : BOOL; (*Get the value of the object continuously while enabled*)
		Index : UINT; (*Index of the object to be read*)
		Subindex : USINT; (*Subindex of the object to be read*)
		pData : UDINT; (*Reference to the data to be read to*)
		DataLen : UDINT; (*Length of the data buffer *)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Reading process finished successfully*)
		Busy : BOOL; (*Still reading*)
		Error : BOOL; (*True if reading process was not successful*)
		ErrorId : WORD; (*Error ID in case of error, dose not include SDO-Abort-Error*)
		ReadLen : UDINT; (*Number of read bytes*)
		SdoAbort : DWORD; (*SDO-Abort-Code in case of an SDO Error *)
	END_VAR
	VAR_IN_OUT
		Sab : SAB_REF; (*Reference to SAB*)
	END_VAR
	VAR
		InternalSdoRead : iDD_SdoReadUniversal_DDS;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_SdoReadUniversal_DDS (*Internal FB to manage sending SDO requests*)
	VAR_INPUT
		Enable : BOOL; (*Get the value of the object continuously while enabled*)
		Index : UINT; (*Index of the object to be read*)
		Subindex : USINT; (*Subindex of the object to be read*)
		pData : UDINT; (*Reference to the data to be read to*)
		DataLen : UDINT; (*Length of the data buffer	 	 *)
		NodeNumber : UINT; (*This needs to be UINT, as the NodeNumber in TwinCAT is UINT*)
		SlotName : STRING[80];
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Reading process finished successfully*)
		Busy : BOOL; (*Still reading*)
		Error : BOOL; (*True if reading process was not successful*)
		ErrorId : WORD; (*Error ID in case of error, dose not include SDO-Abort-Error*)
		ReadLen : UDINT; (*Number of read bytes*)
		SdoAbort : DWORD; (*SDO-Abort-Code in case of an SDO Error *)
	END_VAR
	VAR
		FBStatus : iDD_INTERN_STATUS_DDS; (*Internal index of state machine*)
		EplSDORead_0 : EplSDORead; (*AS library for SDO Reading*)
		IntIndex : UINT; (*Index of the object to be read*)
		IntSubindex : USINT; (*Subindex of the object to be read*)
		IntpData : UDINT; (*Reference to the data to be read to*)
		IntDataLen : UDINT; (*Length of the data buffer*)
		Value : DWORD; (*Internal buffer for data of up to 4 bytes, as the SDO read FB already gives the value while being busy*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_SdoWrite_DDS (*Internal FB to manage sending SDO requests*)
	VAR_INPUT
		Execute : BOOL; (*Writes the value of the object*)
		Index : UINT; (*Index of the object to be written*)
		Subindex : USINT; (*Subindex of the object to be written*)
		pData : UDINT; (*Pointer to the data to be written*)
		DataLen : UDINT; (*Length of the data to be written*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Writing process finished successfully*)
		Busy : BOOL; (*Still writing*)
		Error : BOOL; (*True if writing process was not successful*)
		ErrorId : WORD; (*Error ID in case of error, does not include SDO-Abort-Error*)
		SdoAbort : DWORD; (*SDO-Abort-Code in case of an SDO Error *)
	END_VAR
	VAR_IN_OUT
		Axis : AXIS_REF_DDS; (*Reference to Axis*)
	END_VAR
	VAR
		InternalSdoWrite : iDD_SdoWriteUniversal_DDS;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_SdoWrite_SAB (*Internal FB to manage sending SDO requests*)
	VAR_INPUT
		Execute : BOOL; (*Writes the value of the object*)
		Index : UINT; (*Index of the object to be written*)
		Subindex : USINT; (*Subindex of the object to be written*)
		pData : UDINT; (*Pointer to the data to be written*)
		DataLen : UDINT; (*Length of the data to be written*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Writing process finished successfully*)
		Busy : BOOL; (*Still writing*)
		Error : BOOL; (*True if writing process was not successful*)
		ErrorId : WORD; (*Error ID in case of error, does not include SDO-Abort-Error*)
		SdoAbort : DWORD; (*SDO-Abort-Code in case of an SDO Error *)
	END_VAR
	VAR_IN_OUT
		Sab : SAB_REF; (*Reference to SAB*)
	END_VAR
	VAR
		InternalSdoWrite : iDD_SdoWriteUniversal_DDS;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_SdoWriteUniversal_DDS (*Internal FB to manage sending SDO requests*)
	VAR_INPUT
		Execute : BOOL; (*Writes the value of the object*)
		Index : UINT; (*Index of the object to be written*)
		Subindex : USINT; (*Subindex of the object to be written*)
		pData : UDINT; (*Pointer to the data to be written*)
		DataLen : UDINT; (*Length of the data to be written*)
		NodeNumber : UINT; (*This needs to be UINT, as the NodeNumber in TwinCAT is UINT*)
		SlotName : STRING[80];
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Writing process finished successfully*)
		Busy : BOOL; (*Still writing*)
		Error : BOOL; (*True if Writing process was not successful*)
		ErrorId : WORD; (*Error ID in case of error, does not include SDO-Abort-Error*)
		SdoAbort : DWORD; (*SDO-Abort-Code in case of an SDO Error *)
	END_VAR
	VAR
		FBStatus : iDD_INTERN_STATUS_DDS; (*Internal index of state machine*)
		EplSDOWrite_0 : EplSDOWrite; (*Powerlink - Write function block*)
		ExecuteTrigR_0 : R_TRIG; (*Detects the rising edge of Execute*)
		IntIndex : UINT; (*Index of the object to be written*)
		IntSubindex : USINT; (*Subindex of the object to be written*)
		IntpData : UDINT; (*Reference to the data to be written*)
		IntDataLen : UDINT; (*Length of the data buffer*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION iDD_SetErrorInfo_DDS : BOOL (* Sets the information of the ErrorInfo structure, except for the ErrorID itself *)
	VAR_INPUT
		InstanceID : UDINT;
		NodeNumber : UINT;
	END_VAR
	VAR_IN_OUT
		ErrorInfo : DD_ERROR_DDS;
	END_VAR
END_FUNCTION

FUNCTION iDD_SetMotionRelParams_DDS : USINT (*Internal function only!*) (*Use this function to set several parameters in the axis instance
	Returns the PdoRxId that is used for this PDO command*)
	VAR_INPUT
		InstanceID : UDINT; (*Instance ID to be set*)
		ContextSelector : USINT; (*Context selector to be set*)
		ModeOfOp : SINT; (*Mode of operation to be set*)
		MotionStatus : iDD_MOTIONSTATUS_DDS; (*PLCopen motion state to be set*)
		AppTorqueLimit : UINT; (*Application Torque Limit to be set: Needs to be set in every motion FB*)
	END_VAR
	VAR_IN_OUT
		AxisIntern : iDD_AXIS_REF_INTERN_DDS; (*Reference to the axis*)
	END_VAR
END_FUNCTION

FUNCTION_BLOCK iDD_Semaphore_DDS (*Semaphore functionality*)
	VAR_INPUT
		Execute : BOOL;
		SemAction : iDD_INTERN_SEMAPHORE_DDS;
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Successfully done*)
		Error : BOOL; (*True if anything failed*)
	END_VAR
	VAR
		SemCreate_0 : SemCreate;
		Handle : UDINT;
		ExecuteTrigR_0 : R_TRIG;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_TON_DDS (*Wrapper for the standard TON() function block:
The TON function block implements a switch on delay.*)
	VAR_INPUT
		IN : BOOL; (*Input signal*)
		PT : TIME; (*Delay time*)
	END_VAR
	VAR_OUTPUT
		Q : BOOL; (*Output signal, the rising edge of the input signal is delayed by PT.*)
		ET : TIME; (*Elapsed time*)
	END_VAR
	VAR
		IntPT : TIME;
		TS : DINT;
		IN_OLD : BOOL;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION iDD_BOOL_DDS : BOOL
	VAR_IN_OUT
		in : BOOL;
		out : iDD_INTERN_ARG_DDS;
	END_VAR
END_FUNCTION

FUNCTION iDD_BYTE_DDS : BOOL
	VAR_IN_OUT
		in : BYTE;
		out : iDD_INTERN_ARG_DDS;
	END_VAR
END_FUNCTION

FUNCTION iDD_DINT_DDS : BOOL
	VAR_IN_OUT
		in : DINT;
		out : iDD_INTERN_ARG_DDS;
	END_VAR
END_FUNCTION

FUNCTION iDD_DWORD_DDS : BOOL
	VAR_IN_OUT
		in : DWORD;
		out : iDD_INTERN_ARG_DDS;
	END_VAR
END_FUNCTION

FUNCTION iDD_INT_DDS : BOOL
	VAR_IN_OUT
		in : INT;
		out : iDD_INTERN_ARG_DDS;
	END_VAR
END_FUNCTION

FUNCTION iDD_REAL_DDS : BOOL
	VAR_IN_OUT
		in : REAL;
		out : iDD_INTERN_ARG_DDS;
	END_VAR
END_FUNCTION

FUNCTION iDD_SINT_DDS : BOOL
	VAR_IN_OUT
		in : SINT;
		out : iDD_INTERN_ARG_DDS;
	END_VAR
END_FUNCTION

FUNCTION iDD_STRING_DDS : BOOL
	VAR_IN_OUT
		in : STRING[80];
		out : iDD_INTERN_ARG_DDS;
	END_VAR
END_FUNCTION

FUNCTION iDD_UDINT_DDS : BOOL
	VAR_IN_OUT
		in : UDINT;
		out : iDD_INTERN_ARG_DDS;
	END_VAR
END_FUNCTION

FUNCTION iDD_UINT_DDS : BOOL
	VAR_IN_OUT
		in : UINT;
		out : iDD_INTERN_ARG_DDS;
	END_VAR
END_FUNCTION

FUNCTION iDD_USINT_DDS : BOOL
	VAR_IN_OUT
		in : USINT;
		out : iDD_INTERN_ARG_DDS;
	END_VAR
END_FUNCTION

FUNCTION iDD_WORD_DDS : BOOL
	VAR_IN_OUT
		in : WORD;
		out : iDD_INTERN_ARG_DDS;
	END_VAR
END_FUNCTION

FUNCTION iDD_memset_DDS : UDINT (*Wrapper for memset: Fills the memory area with specific values
	Returns: Address of the destination*)
	VAR_INPUT
		pDest : UDINT; (*Address of destination memory*)
		value : USINT; (*Initialization value*)
		length : UDINT; (*Number of bytes to be initialized*)
	END_VAR
END_FUNCTION

FUNCTION iDD_memcpy_DDS : UDINT (*Wrapper for memcpy: Copies the memory area
	Returns: Address of the destination*)
	VAR_INPUT
		pDest : UDINT; (*Address of destination memory*)
		pSrc : UDINT; (*Address of data to be copied*)
		length : UDINT; (*Number of bytes to be copied*)
	END_VAR
END_FUNCTION

FUNCTION iDD_memmove_DDS : UDINT (*Wrapper for memmove: Copies the memory area (even if the memory areas overlap)
	Returns: Address of the destination*)
	VAR_INPUT
		pDest : UDINT; (*Address of destination memory*)
		pSrc : UDINT; (*Address of data to be copied*)
		length : UDINT; (*Number of bytes to be copied*)
	END_VAR
END_FUNCTION

FUNCTION iDD_GetTaskCycleTime_DDS : REAL (*Gets the task cycle time in s*)
	VAR
		RTInfo_0 : RTInfo;
	END_VAR
END_FUNCTION

FUNCTION iDD_DEC_TO_INT_DDS : INT
	VAR_INPUT
		DEC : STRING[20];
	END_VAR
	VAR
		pt : REFERENCE TO BYTE;
		i : INT;
		X : BYTE;
		sign : BOOL;
		stop : INT;
	END_VAR
END_FUNCTION

FUNCTION iDD_BIT_OF_DWORD_DDS : BOOL
	VAR_INPUT
		in : DWORD;
		n : INT;
	END_VAR
END_FUNCTION

FUNCTION iDD_BUFFER_SEARCH_DDS : INT
	VAR_INPUT
		PT : REFERENCE TO ARRAY[0..32767] OF BYTE;
		SIZE : INT;
		STR : DD_MAX_STRING_DDS;
		POS : INT;
		IGN : BOOL;
	END_VAR
	VAR
		ps : REFERENCE TO ARRAY[0..iDD_MAX_STRING_LENGTH] OF BYTE;
		chx : BYTE;
		i : INT;
		end : INT;
		k : INT;
		lx : INT;
	END_VAR
END_FUNCTION

FUNCTION_BLOCK iDD_BUFFER_TO_STRING_DDS
	VAR_INPUT
		PT : REFERENCE TO ARRAY[0..32767] OF BYTE;
		SIZE : UINT;
		START : UINT;
		STOP : UINT;
	END_VAR
	VAR_IN_OUT
		sOut : DD_MAX_STRING_DDS;
	END_VAR
	VAR
		ps : REFERENCE TO BYTE;
		i : UINT;
		stp : UINT;
		sta : UINT;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_DINT_TO_DECSTR_DDS
	VAR_INPUT
		in : DINT;
		iPrecision : INT;
	END_VAR
	VAR_IN_OUT
		sOut : DD_MAX_STRING_DDS;
	END_VAR
	VAR
		DWORD_TO_DECSTR_0 : iDD_DWORD_TO_DECSTR_DDS;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_DWORD_TO_BINSTR_DDS
	VAR_INPUT
		in : DWORD;
		iPrecision : INT;
	END_VAR
	VAR_IN_OUT
		sOut : DD_MAX_STRING_DDS;
	END_VAR
	VAR
		bits : ARRAY[0..32] OF BYTE; (* array of ASCII characters (inclusive null delimiter) *)
		iSig : INT; (* number of significant bits *)
		iPad : INT; (* number of padding zeros *)
		i : INT;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_DWORD_TO_DECSTR_DDS
	VAR_INPUT
		in : DWORD;
		iPrecision : INT;
	END_VAR
	VAR_IN_OUT
		sOut : DD_MAX_STRING_DDS;
	END_VAR
	VAR
		dec : ARRAY[0..11] OF BYTE; (* array of ASCII characters (inclusive null delimiter) *)
		iSig : INT; (* number of significant nibbles *)
		iPad : INT; (* number of padding zeros *)
		i : INT;
		divider : UDINT := 1000000000;
		number : UDINT;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_DWORD_TO_HEXSTR_DDS
	VAR_INPUT
		in : DWORD;
		iPrecision : INT;
		bLoCase : BOOL := FALSE; (*Default: use "ABCDEF", if TRUE use "abcdef" characters. *)
	END_VAR
	VAR_IN_OUT
		sOut : DD_MAX_STRING_DDS;
	END_VAR
	VAR
		hex : ARRAY[0..8] OF BYTE; (* array of ASCII characters (inclusive null delimiter) *)
		iSig : INT; (* number of significant nibbles *)
		iPad : INT; (* number of padding zeros *)
		i : INT;
	END_VAR
	VAR CONSTANT
		FORMAT_HEXASC_CODES : ARRAY[0..1,0..15] OF BYTE := [16#30,16#31,16#32,16#33,16#34,16#35,16#36,16#37,16#38,16#39,16#61,16#62,16#63,16#64,16#65,16#66,16#30,16#31,16#32,16#33,16#34,16#35,16#36,16#37,16#38,16#39,16#41,16#42,16#43,16#44,16#45,16#46]; (*upper case*) (* ASCII to decimal and decimal digits to ASCII codes *)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_DWORD_TO_OCTSTR_DDS
	VAR_INPUT
		in : DWORD;
		iPrecision : INT;
	END_VAR
	VAR_IN_OUT
		sOut : DD_MAX_STRING_DDS;
	END_VAR
	VAR
		oct : ARRAY[0..12] OF BYTE; (* array of ASCII characters (inclusive null delimiter) *)
		iSig : INT; (* number of significant nibbles *)
		iPad : INT; (* number of padding zeros *)
		i : INT;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION iDD_EXPN_DDS : REAL
	VAR_INPUT
		x : REAL;
		n : INT;
	END_VAR
	VAR
		sign : BOOL;
	END_VAR
END_FUNCTION

FUNCTION iDD_FINDB_DDS : INT
	VAR_INPUT
		str1 : DD_MAX_STRING_DDS;
		str2 : DD_MAX_STRING_DDS;
	END_VAR
	VAR
		pos : INT;
		length : INT;
	END_VAR
END_FUNCTION

FUNCTION iDD_FLOAT_TO_REAL_DDS : REAL
	VAR_INPUT
		flt : STRING[20];
	END_VAR
	VAR
		pt : REFERENCE TO ARRAY[1..20] OF BYTE;
		i : INT;
		x : USINT;
		sign : INT := 1;
		stop : INT;
		tmp : DINT;
		d : INT;
	END_VAR
END_FUNCTION

FUNCTION_BLOCK iDD_GetFloatRec_DDS (*Double precision (LREAL )
	If exponent=2047 and fraction is nonzero, then x = Nan ("Not a number") 
	If exponent=2047 and fraction is zero and sign is 1, then x = "-Infinity" 
	If exponent=2047 and fraction is zero and sign is 0, then x = "Infinity"*)
	VAR_INPUT
		fVal : LREAL;
		iPrecision : INT;
		bRound : BOOL;
	END_VAR
	VAR_IN_OUT
		fOut : iDD_INTERN_FLOAT_REC_DDS;
	END_VAR
	VAR
		ptrDouble : REFERENCE TO ARRAY[0..1] OF DWORD;
		fValue : LREAL;
		fBegin : LREAL;
		nBegin : INT;
		fDiv : LREAL;
		nDigit : INT := 0;
		fMaxPrecision : LREAL;
		i : INT;
		nLastDecDigit : INT;
		nDig : BYTE;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_LREAL_TO_FMTSTR_DDS (*Signed value having the form [  ]dddd.dddd, where dddd is one or more decimal digits.
	The number of digits before the decimal point depends on the magnitude of the number, 
	and the number of digits after the decimal point depends on the requested precision.
	Sign appears only for negative values.  
	If the given value is positive infinity, the resulting string is '#INF'. If the given value is negative infinity, the resulting string is '-#INF'.
	IF the given value is a NAN (NOT-a-number), the resulting string  is '#QNAN' OR '-#QNAN'.
	If the length of the resulting string is greater than the max. length of the return value, the resulting string is '#OVF' or '-#OVF'.*)
	VAR_INPUT
		in : LREAL;
		iPrecision : INT;
		bRound : BOOL;
	END_VAR
	VAR_IN_OUT
		sOut : DD_MAX_STRING_DDS;
	END_VAR
	VAR
		rec : iDD_INTERN_FLOAT_REC_DDS;
		pOut : REFERENCE TO BYTE;
		iStart : INT;
		iEnd : INT;
		i : INT;
		F_GetFloatRec_0 : iDD_GetFloatRec_DDS;
		sExp : DD_MAX_STRING_DDS;
		iDD_DINT_TO_DECSTR_DDS_0 : iDD_DINT_TO_DECSTR_DDS;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION iDD_TO_UPPER_DDS : BYTE
	VAR_INPUT
		in : BYTE;
	END_VAR
END_FUNCTION

FUNCTION_BLOCK iDD_TRIM_DDS
	VAR_IN_OUT
		sOut : DD_MAX_STRING_DDS;
	END_VAR
	VAR
		pChar : REFERENCE TO BYTE;
		pStr : REFERENCE TO DD_MAX_STRING_DDS;
		n : UDINT;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_XML_READER_DDS
	VAR_IN_OUT
		CTRL : iDD_INTERN_XML_CONTROL_DDS;
		BUF : ARRAY[0..iDD_MAX_BUFFER_INDEX] OF BYTE;
	END_VAR
	VAR
		index : INT;
		index2 : INT;
		stop : INT;
		mode : INT;
		command : WORD;
		c : BYTE;
		s1 : INT;
		e1 : INT;
		path_overflow : BOOL;
		empty_tag : BOOL;
		s_code : STRING[10];
		e_code : STRING[10];
		watchdog : TON;
		iDD_BUFFER_TO_STRING_DDS_0 : iDD_BUFFER_TO_STRING_DDS; (*LOG_MSG : LOG_MSG;
		_debug_enable : BOOL := TRUE;*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_FileDelete_DDS (*This FB deletes a file from the desired device*)
	VAR_INPUT
		Execute : BOOL; (*Execute*)
		pDevice : UDINT; (*Pointer to the device name on which the file is located*)
		TargetFile : STRING[80]; (*Path of file and target file name on the drive like "DSP/fs/test.bin"*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*The file is opened correctly*)
		Busy : BOOL; (*The FB is not finished and new output values are to be expected*)
		Error : BOOL; (*Signals that an error has occurred within the FB*)
		ErrorId : WORD; (*Error information*)
	END_VAR
	VAR
		IntpDevice : UDINT; (*Pointer to the device name on which the file is located*)
		IntTargetFile : STRING[80]; (*Path of file and target file name on the drive like "DSP/fs/test.bin"*)
		ExecuteTrigR_0 : R_TRIG; (*Rising edge of Execute*)
		FBStatus : iDD_INTERN_STATUS_DDS; (*Status value of state machine of FB*)
		FileDelete_0 : FileDelete; (*This FBK deletes a file from the desired device*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_SdoRead_ACM (*Internal FB to manage sending SDO requests*)
	VAR_INPUT
		Enable : BOOL; (*Get the value of the object continuously while enabled*)
		Index : UINT; (*Index of the object to be read*)
		Subindex : USINT; (*Subindex of the object to be read*)
		pData : UDINT; (*Reference to the data to be read to*)
		DataLen : UDINT; (*Length of the data buffer *)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Reading process finished successfully*)
		Busy : BOOL; (*Still reading*)
		Error : BOOL; (*True if reading process was not successful*)
		ErrorId : WORD; (*Error ID in case of error, dose not include SDO-Abort-Error*)
		ReadLen : UDINT; (*Number of read bytes*)
		SdoAbort : DWORD; (*SDO-Abort-Code in case of an SDO Error *)
	END_VAR
	VAR_IN_OUT
		Acm : ACM_REF; (*Reference to ACM*)
	END_VAR
	VAR
		InternalSdoRead : iDD_SdoReadUniversal_DDS;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_SdoWrite_ACM (*Internal FB to manage sending SDO requests*)
	VAR_INPUT
		Execute : BOOL; (*Writes the value of the object*)
		Index : UINT; (*Index of the object to be written*)
		Subindex : USINT; (*Subindex of the object to be written*)
		pData : UDINT; (*Pointer to the data to be written*)
		DataLen : UDINT; (*Length of the data to be written*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Writing process finished successfully*)
		Busy : BOOL; (*Still writing*)
		Error : BOOL; (*True if writing process was not successful*)
		ErrorId : WORD; (*Error ID in case of error, does not include SDO-Abort-Error*)
		SdoAbort : DWORD; (*SDO-Abort-Code in case of an SDO Error *)
	END_VAR
	VAR_IN_OUT
		Acm : ACM_REF; (*Reference to ACM*)
	END_VAR
	VAR
		InternalSdoWrite : iDD_SdoWriteUniversal_DDS;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_SdoRead_DAM (*Internal FB to manage sending SDO requests*)
	VAR_INPUT
		Enable : BOOL; (*Get the value of the object continuously while enabled*)
		Index : UINT; (*Index of the object to be read*)
		Subindex : USINT; (*Subindex of the object to be read*)
		pData : UDINT; (*Reference to the data to be read to*)
		DataLen : UDINT; (*Length of the data buffer *)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Reading process finished successfully*)
		Busy : BOOL; (*Still reading*)
		Error : BOOL; (*True if reading process was not successful*)
		ErrorId : WORD; (*Error ID in case of error, dose not include SDO-Abort-Error*)
		ReadLen : UDINT; (*Number of read bytes*)
		SdoAbort : DWORD; (*SDO-Abort-Code in case of an SDO Error *)
	END_VAR
	VAR_IN_OUT
		Dam : DAM_REF; (*Reference to SAB*)
	END_VAR
	VAR
		InternalSdoRead : iDD_SdoReadUniversal_DDS;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_SdoWrite_DAM (*Internal FB to manage sending SDO requests*)
	VAR_INPUT
		Execute : BOOL; (*Writes the value of the object*)
		Index : UINT; (*Index of the object to be written*)
		Subindex : USINT; (*Subindex of the object to be written*)
		pData : UDINT; (*Pointer to the data to be written*)
		DataLen : UDINT; (*Length of the data to be written*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Writing process finished successfully*)
		Busy : BOOL; (*Still writing*)
		Error : BOOL; (*True if writing process was not successful*)
		ErrorId : WORD; (*Error ID in case of error, does not include SDO-Abort-Error*)
		SdoAbort : DWORD; (*SDO-Abort-Code in case of an SDO Error *)
	END_VAR
	VAR_IN_OUT
		Dam : DAM_REF; (*Reference to SAB*)
	END_VAR
	VAR
		InternalSdoWrite : iDD_SdoWriteUniversal_DDS;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_SdoRead_PSM (*Internal FB to manage sending SDO requests*)
	VAR_INPUT
		Enable : BOOL; (*Get the value of the object continuously while enabled*)
		Index : UINT; (*Index of the object to be read*)
		Subindex : USINT; (*Subindex of the object to be read*)
		pData : UDINT; (*Reference to the data to be read to*)
		DataLen : UDINT; (*Length of the data buffer *)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Reading process finished successfully*)
		Busy : BOOL; (*Still reading*)
		Error : BOOL; (*True if reading process was not successful*)
		ErrorId : WORD; (*Error ID in case of error, dose not include SDO-Abort-Error*)
		ReadLen : UDINT; (*Number of read bytes*)
		SdoAbort : DWORD; (*SDO-Abort-Code in case of an SDO Error *)
	END_VAR
	VAR_IN_OUT
		Psm : PSM_REF; (*Reference to PSM*)
	END_VAR
	VAR
		InternalSdoRead : iDD_SdoReadUniversal_DDS;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK iDD_SdoWrite_PSM (*Internal FB to manage sending SDO requests*)
	VAR_INPUT
		Execute : BOOL; (*Writes the value of the object*)
		Index : UINT; (*Index of the object to be written*)
		Subindex : USINT; (*Subindex of the object to be written*)
		pData : UDINT; (*Pointer to the data to be written*)
		DataLen : UDINT; (*Length of the data to be written*)
	END_VAR
	VAR_OUTPUT
		Done : BOOL; (*Writing process finished successfully*)
		Busy : BOOL; (*Still writing*)
		Error : BOOL; (*True if writing process was not successful*)
		ErrorId : WORD; (*Error ID in case of error, does not include SDO-Abort-Error*)
		SdoAbort : DWORD; (*SDO-Abort-Code in case of an SDO Error *)
	END_VAR
	VAR_IN_OUT
		Psm : PSM_REF; (*Reference to PSM*)
	END_VAR
	VAR
		InternalSdoWrite : iDD_SdoWriteUniversal_DDS;
	END_VAR
END_FUNCTION_BLOCK

FUNCTION iDD_CommunicationReady_PSM : BOOL (*Checks if there is communication to the PSM*)
	VAR_IN_OUT
		Psm : PSM_REF;
	END_VAR
END_FUNCTION

FUNCTION iDD_CommunicationReady_DAM : BOOL (*Checks if there is communcation to the DAM*)
	VAR_IN_OUT
		Dam : DAM_REF;
	END_VAR
END_FUNCTION

FUNCTION iDD_CommunicationReady_ACM : BOOL (*Checks if there is communication to the ACM*)
	VAR_IN_OUT
		Acm : ACM_REF;
	END_VAR
END_FUNCTION
