(********************************************************************
 * COPYRIGHT -- Danfoss A/S
 ********************************************************************
 * Library: DDS_Intern
 * File: DDS_Intern.typ
 * Author: DESC0724
 * Created: Febuary 03, 2014
 ********************************************************************
 * Data types of library DDS_Intern
 *********************************************************************)

TYPE
	iDD_AXIS_REF_INTERN_DDS :
	STRUCT
		MotionStatus : 					iDD_MOTIONSTATUS_DDS; 					(*Identifies the state of the PLCopen state diagram*)
		Ds402State : 						iDD_DS402_STATE_DDS; 						(*State of DS402*)
		Ds402State_Old : 					iDD_DS402_STATE_DDS; 						(*State of DS402 of previous cycle*)
	
	(*administrative variables*)
		iDD_CycleInfo_DDS_0 : 			iDD_CycleInfo_DDS; 							(*Information of times of calling the current cylce "CycleCounter"*)
		ActiveInstance : 					UDINT; 										(*Instance ID of the instance that is currently in control of this axis*)
		ActiveInstanceActivated :			USINT; 										(*Indicates the PDO counter that was used when the handshaking has been started*)
		ActiveInstanceBuffered : 			UDINT;										(*Instance ID of the instance for buffered commands*)
		ActiveInstanceBufferedActivated :	USINT;										(*Indicates the PDO counter that was used when the handshaking has been started for the buffered command*)
		ActiveInstanceDone :				UDINT;										(*Holds the instance ID of the completed command*)
		ActiveTouchProbeInstance : 		ARRAY[1..2]OF UDINT; 						(*ID of the instance that is currently in control of the touch probe input 1 or 2 of the axis*)
		IsBlocked : 						BOOL; 										(*General motion blocking State set by IsMeasuring , IsStopping , IsHoming aso.*)
		IsStopping : 						BOOL;										(*Case of blocking set by MC_Stop*)
		IsFirmwareUpdating : 				BOOL; 										(*Case of Blocking set by DD_FirmwareUpdate*)
		IsHoming : 						BOOL; 										(*Case of Blocking set by MC_Home*)
		NewCommand : 					BOOL; 										(*Command to send (RPDO) to drive/SAB on FB request *)
		FosSemaphore : 					iDD_Semaphore_DDS; 						(*Initialization of the FoS semaphore *)
		InitDone : 						BOOL; 										(*One time initialization of axis structure *)
		CommandErrorRaised :			BOOL;										(*Detects a rising edge of the Command error bit in the Statusword*)
	
	(*PDO administrative variables*)
		PdoRxId : 						USINT; 										(*Index of RPDO: Used for handshaking of commands*)
		TriggerBit : 						BOOL;										(*Trigger bit*)
		ContextSelector : 					USINT; 										(*Identifier of context selector for RPDO*)
		AdrCounter: 						USINT; 										(*Current postion index in the TPDO array for unpacking resp. RPDO array for packing the variables*)
	
	(*Single parameters for RPDO; fixed part*)
		Controlword : 						UINT; 										(*RPDO-References 0x6040*)
		Controlword_Old : 					UINT; 										(*Controlword of last cycle*)
		ModesOfOp : 						SINT := iDD_MoO_DS402_ProfilePosition; 		(*RPDO-References 0x6060*)
		ModesOfOp_Old : 					SINT := iDD_MoO_DS402_ProfilePosition; 		(*Mode of Operation of last cycle*)
		PdoRxIdCs : 						USINT; 										(*RPDO contains 2 Bits for RPDO-ID and 4 Bits for Context Selector, a reserved one and the trigger bit*)
	
	 (*Single parameters for RPDO; are packed into the RPDO, depending on the context selector*)
		TargetPosition : 					DINT; 										(*RPDO-References 0x607A*)
		ProfileVelocity : 					UDINT; 										(*RPDO-References 0x6081*)
		ProfileAcceleration : 				UDINT; 										(*RPDO-References 0x6083*)
		ProfileDeceleration : 				UDINT; 										(*RPDO-References 0x6084*)
		PositioningOptionCode : 			UINT; 										(*RPDO-References 0x60F2*)
		AppTorqueLimit : 					UINT := 16#FFFF; 								(*RPDO-References 0x2053*)
		TargetVelocity : 					DINT; 										(*RPDO-References 0x60FF*)
		SuperimposedControl : 			USINT; 										(*RPDO-References 0xXXXX*)
		SuperimposedPosition : 			DINT; 										(*RPDO-References 0xXXXX*)
		SuperimposedVelocity : 			DINT; 										(*RPDO-References 0xXXXX*)
		SuperimposedAcceleration : 		UDINT; 										(*RPDO-References 0xXXXX*)
		SuperimposedDeceleration : 		UDINT; 										(*RPDO-References 0xXXXX*)
		TargetTorque : 					INT; 										(*RPDO-References 0x6071*)
		TorqueSlope : 					UDINT; 										(*RPDO-References 0x6087*)
		MaxMotorSpeed : 					UDINT; 										(*RPDO-References 0x6080*)
		PositionGuideValue : 				UDINT; 										(*RPDO-References 0x2060*)
		GearRatioNumerator : 				DINT; 										(*RPDO-References 0xXXXX*)
		GearRatioDenominator : 			DINT; 										(*RPDO-References 0xXXXX*)
		MasterSyncPosition : 				DINT; 										(*RPDO-References 0xXXXX*)
		SlaveSyncPosition : 				DINT; 										(*RPDO-References 0xXXXX*)
		MasterStartDistance : 				UDINT; 										(*RPDO-References 0xXXXX*)
		PhasingControlAndStatus : 			USINT; 										(*RPDO-References 0xXXXX*)
		PhaseShift : 						DINT; 										(*RPDO-References 0xXXXX*)
		PhasingVelocity : 					REAL; 										(*RPDO-References 0xXXXX*)
		PhasingAcceleration : 				REAL; 										(*RPDO-References 0xXXXX*)
		PhasingDeceleration : 				REAL; 										(*RPDO-References 0xXXXX*)
		VelocityGuideValue : 				REAL; 										(*RPDO-References 0x2064*)
		CamMasterOffset : 				DINT; 										(*RPDO-References 0x3806*)
		CamSlaveOffset : 					REAL; 										(*RPDO-References 0x3807*)
		CamProfileSelector : 				UINT; 										(*RPDO-References 0x3804*)
		CamBlendingDistance : 			UDINT; 										(*RPDO-References 0x380A*)
		CamCmdFree : 					BOOL;										(*Indicates if there has already been a CAM command during this PLC cycle*)
		CamControlCode : 				UINT; 										(*RPDO-References 0x3800,01*)
		CamControlParam1 : 				UINT; 										(*RPDO-References 0x3800,02*)
		CamControlParam2 : 				UINT; 										(*RPDO-References 0x3800,03*)
		CamControlParam3 : 				UINT; 										(*RPDO-References 0x3800,04*)
	
	(*TPDO parameters*)
		ModesOfOpDisplay : 				SINT; 										(*TPDO-References 0x6061*)
		ModesOfOpDisplay_Old: 			SINT; 										(*TPDO-References 0x6061 of last cycle*)
		Statusword : 						UINT; 										(*TPDO-References 0x6041*)
		Statusword_Old :					UINT;										(*Statusword of last cycle*)
		PdoTxId : 						USINT; 										(*TPDO-References 0x5041*)
		MotionInputStatus : 				INT; 										(*TPDO-References 0x2006*)
		PositionActualValue : 				DINT; 										(*TPDO-References 0x6064*)
		VelocityActualValue : 				DINT; 										(*TPDO-References 0x606C*)
		TorqueActualValue : 				INT; 										(*TPDO-References 0x6077*)
		OverlayingMotionStatus : 			UINT; 										(*TPDO-References 0x2005*)
		CoveredSuperImposedDistance :	UDINT; 										(*TPDO-References 0xXXXX*)
		AbsolutPhaseShift : 				UDINT; 										(*TPDO-References 0xXXXX*)
		CamProfileStatus : 				UINT; 										(*TPDO-References 0x3804*)
		CamProfileStatus_Old : 			UINT; 										(*TPDO-References 0x3804 of last cycle*)
		CamStatusCode : 					UINT; 										(*TPDO-References 0x3801,01*)
		CamStatusParam1 : 				UINT; 										(*TPDO-References 0x3801,02*)
		CamStatusParam2 : 				UINT; 										(*TPDO-References 0x3801,03*)
		CamStatusParam3 : 				UINT; 										(*TPDO-References 0x3801,04*)
	END_STRUCT;
	iDD_DS402_STATE_DDS :
	( (*States of the DS402 state machine*)
		ds402_Unknown_DDS, 	(*State is unknown*)
		ds402_NtRdySwOn_DDS, 	(*Not ready to switch on*)
		ds402_SwthOnDis_DDS, 	(*Switch on disabled*)
		ds402_RdyToSwOn_DDS, 	(*Ready to switch on*)
		ds402_SwitchedOn_DDS, 	(*Switched on*)
		ds402_OpEnabled_DDS, 	(*Operation enabled*)
		ds402_QuickstActv_DDS, 	(*Quick stop active*)
		ds402_FaultReact_DDS, 	(*Fault reaction active*)
		ds402_Fault_DDS 			(*Fault*)
	);
	iDD_FileOpenMode_DDS :
	(
		iReadOnly,
		iWriteOnly
	);
	iDD_INTERN_ARG_DDS :
	STRUCT
		eType : iDD_INTERN_ARGTYPE_DDS := iARGTYPE_UNKNOWN; 	(*Argument data type*)
		cbLen : UDINT := 0; 											(*Argument data byte length*)
		pData : UDINT := 0; 											(*Pointer to argument data*)
	END_STRUCT;
	iDD_INTERN_ARGTYPE_DDS :
	(
		iARGTYPE_UNKNOWN := 0,
		iARGTYPE_BYTE,
		iARGTYPE_WORD,
		iARGTYPE_DWORD,
		iARGTYPE_REAL,
		iARGTYPE_LREAL,
		iARGTYPE_SINT,
		iARGTYPE_INT,
		iARGTYPE_DINT,
		iARGTYPE_USINT,
		iARGTYPE_UINT,
		iARGTYPE_UDINT,
		iARGTYPE_STRING, 		(* string of type T_MaxString! *)
		iARGTYPE_BOOL,
		iARGTYPE_BIGTYPE, 		(* byte buffer *)
		iARGTYPE_ULARGE, 		(* unsigned 64 bit ingeger (T_ULARGE_INTEGER) *)
		iARGTYPE_UHUGE, 		(* unsigned 128 bit integer (T_UHUGE_INTEGER) *)
		iARGTYPE_LARGE,		(* signed 64 bit integer (T_LARGE_INTEGER) *)
		iARGTYPE_HUGE 		(* signed 128 bit integer (T_HUGE_INTEGER) *)
	);
	iDD_INTERN_BASIC_DDS :
	STRUCT
		InstanceID : 	UDINT; 							(*Unique identifier for the FB instance. Is assigned at first activation*)
		FBStatus : 	iDD_INTERN_STATUS_DDS; 		(* Status value of state machine of FB*)
	END_STRUCT;
	iDD_INTERN_CAM_CMD_DDS :
	STRUCT
		InstanceID : 			UDINT; 						(*Unique identifier for the FB instance. Is assigned at first activation*)
		FBStatus : 			iDD_INTERN_STATUS_DDS; 	(* Status value of state machine of FB*)
		ExecuteTrigR_0 :		R_TRIG; 						(* Positive edge of Execute *)
		IntControlCode : 		UINT;
		IntParam1 : 			UINT;
		IntParam2 : 			UINT;
		IntParam3 : 			UINT;
	END_STRUCT;
	iDD_INTERN_CAM_TABLE_SEL_DDS :
	STRUCT
		InstanceID : 				UDINT; 							(*Unique identifier for the FB instance. Is assigned at first activation*)
		FBStatus : 				iDD_INTERN_STATUS_DDS; 		(* Status value of state machine of FB*)
		FileTransfer_0 : 			iDD_FileTransfer_DDS; 			(*Transfers the CAM profile*)
		SdoWrite_0 : 				iDD_SdoWrite_DDS; 				(*Write the CAM configuration*)
		SdoRead_0 : 				iDD_SdoRead_DDS;
		ExecuteTrigR_0 : 			R_TRIG; 							(* Positive edge of Execute *)
		IntMasterAbsolute : 		BOOL;
		IntSlaveAbsolute : 			BOOL;
		IntCyclic : 				BOOL;
		IntpDevice : UDINT;
		IntCamTable : 			STRING[80];
		IntPatternFile : 			STRING[80];
		IntCamTableID : 			USINT;
		DDCamTableSelStatus : 	USINT;
		CamConfiguration : 		UINT;
		ParsingState : 			WORD;
		ParsingErrorInfo: 			DWORD;
		TargetFileName : 			STRING[80];
	END_STRUCT;
	iDD_INTERN_FILE_HANDLE_DDS :
	STRUCT
		Handle : 		UDINT;		(*File handle coming from the File open FB*)
		Offset : 		UDINT;		(*Offset inside the file*)
	END_STRUCT;
	iDD_INTERN_FILETRANS_STAT_DDS :
	(
		ddFile_FileOpen_DDS, 			(*Open file that shall be transferred*)
		ddFile_WriteTargetPath_DDS,		(*Write target path to drive*)
		ddFile_StartTransferCmd_DDS, 		(*Write start file transfer command to drive*)
		ddFile_ReadFile_DDS, 			(*Read a part of the file to the buffer*)
		ddFile_WriteBuffer_DDS,			(*Write the content of the buffer to the drive*)
		ddFile_FinishTransferCmd_DDS, 	(*Write abort or completed file transfer command to drive*)
		ddFile_CloseFile_DDS, 			(*Close file on PLC*)
		ddFile_WaitDelay_DDS, 			(*Wait when the drive is not ready to accept new data*)
		ddFile_FileDelete_DDS			(*Deletes a file from the desired device*)
	);
	iDD_INTERN_FLOAT_REC_DDS :
	STRUCT
		bNeg : 	BOOL := FALSE; 			(*TRUE if the number is negative, FALSE if the number is zero or positive.*)
		iExp : 	INT := 0; 					(*Set to -32768 if number is NAN. Set to 32767 if number is +INF or -INF.*)
		cDigits : 	ARRAY[0..iDD_FLOATREC_MAX_DIGITS] OF BYTE := 16#30,iDD_FLOATREC_MAX_DIGITS(16#30); (*this should be right*)
	(*	cDigits[ 0 ]  rounding operation overflow result
		cDigits[ 1 ]
		"." dot sign
		cDigits[ 2 ]
		cDigits[ 3 ]
		cDigits[ n ]
		If iExp > 0 => shift dot sign to the right
		else if iExp < 0 => shift dot sign to the left
		Add zeros if ABS(iExp) > FORMAT_DOUBLE_MAX_PRECISION	*)
	END_STRUCT;
	iDD_INTERN_FORMATPARAMS_DDS :
	STRUCT
		bPercent : 			BOOL;
		bFlags : 				BOOL;
		bWidth : 				BOOL;
		bDot : 				BOOL;
		bPrecision : 			BOOL;
		bType : 				BOOL; 	(* Flags field *)
		bAlign : 				BOOL; 	(* Default :Right align *)
		bSign :				BOOL; 	(* Default: Sign only for negative values *)
		bNull : 				BOOL; 	(* Default: No padding *)
		bBlank : 				BOOL; 	(* Default: No blanks*)
		bHash : 				BOOL; 	(* Default: No blanks *)
		iWidth : 				INT;
		iPrecision : 			INT;
		bWidthAsterisk : 		BOOL;
		bPrecisionAsterisk : 	BOOL;
		eType : 				iDD_INTERN_TYPEFIELDPARA_DDS; (* format type parameter *)
	END_STRUCT;
	iDD_INTERN_GUIDEVALUE_DDS :
	STRUCT
		InstanceID : 			UDINT; 						(*Unique identifier for the FB instance. Is assigned at first activation*)
		FBStatus : 			iDD_INTERN_STATUS_DDS; 	(* Status value of state machine of FB*)
		TaskCycleTime : 		REAL; 						(*Task cycle time given in microseconds*)
		Delta : 				REAL; 						(*Delta velocity value per cycle*)
	END_STRUCT;
	iDD_INTERN_HASH_PREFIX_DDS :
	(
		iHASHPREFIX_IEC := 0,	(* 2#, 8#, 16# *)
		iHASHPREFIX_STDC 		(* 0 for octal type, 0x, 0X for hex else none  *)
		);
	iDD_INTERN_HOME_DDS :
	STRUCT
		ExecuteTrigR_0 : 				R_TRIG; 								(* Positive edge of Execute *)
		InstanceID : 					UDINT; 								(*Unique identifier for the FB instance. Is assigned at first activation*)
		FBStatus : 					iDD_INTERN_STATUS_DDS; 			(* Status value of state machine of FB*)
		DDHomeStatus : 				iDD_INTERN_HOME_STATUS_DDS;
		PdoIdCounter : 				USINT;
		SdoRead_0 : 					iDD_SdoRead_DDS; 					(* Instance of SDO Reading function block*)
		SdoWrite_0 : 					iDD_SdoWrite_DDS; 					(* Instance of SDO Writing function block*)
		StartOnDisabledAllowed : 		BOOL;
		PLCopenStateOk : 			BOOL; 								(*Precondition of PLCopen state is ok*)
		IntPosition : 					DINT;
		IntMode : 					DD_HOME_MODE_DDS;
		IntVelocity : 					UDINT;
		IntVelocityOut : 				UDINT;
		IntAcceleration : 				UDINT;
		IntDeceleration : 				UDINT;
		IntTorqueLimit : 				UINT;
		IntBlockedVelocity : 			UDINT;
		IntBlockedDuration : 			UINT;
		IntDistanceLimit : 				UDINT;
		IntTimeLimit : 					TIME;
		MotTimer: 					USINT; 								(*Timer to count the cycles until the mode of operation changes*)
		Timer : 						iDD_TON_DDS;
	END_STRUCT;
	iDD_INTERN_HOME_STATUS_DDS :
	(
		ddHome_WriteMethode_DDS,
		ddHome_WritePosition_DDS,
		ddHome_WriteVelocity_DDS,
		ddHome_WriteVelocityOut_DDS,
		ddHome_WriteAcceleration_DDS,
		ddHome_WriteDeceleration_DDS,
		ddHome_WriteBlockSpeed_DDS,
		ddHome_WriteBlockTime_DDS,
		ddHome_WriteLimitDistance_DDS,
		ddHome_Command_DDS,
		ddHome_Evaluation_DDS
	);
	iDD_INTERN_INERTIA_DDS :
	STRUCT
		InstanceID : 				UDINT; 								(*Unique identifier for the FB instance. Is assigned at first activation*)
		SdoRead_0 : 				iDD_SdoRead_DDS; 					(*Instance of SDO Reading function block*)
		ExecuteTrigR_0 : 			R_TRIG; 								(*Positive edge of Execute *)
		FBStatus : 				iDD_INTERN_STATUS_DDS; 			(*Status value of state machine of FB*)
		DDInertiaStatus : 			iDD_INTERN_INERTIA_STATUS_DDS; 	(*Internal state machine variable*)
		SdoWrite_0 :				iDD_SdoWrite_DDS; 					(* Instance of SDO Writing function block*)
		PdoIdCounter : 			USINT;
		IntVelocity : 				UDINT;
		IntTorque : 				UINT;
		MotTimer : 				USINT; 								(*Timer to count the cycles until the mode of operation changes*)
	END_STRUCT;
	iDD_INTERN_INERTIA_STATUS_DDS :
	(
		ddInertia_Default_DDS,
		ddInertia_WriteAccTorque_DDS,
		ddInertia_WriteVelocity_DDS,
		ddInertia_StartMeasure_DDS,
		ddInertia_Evaluation_DDS,
		ddInertia_WaitStandstill_DDS
	);
	iDD_INTERN_LOAD_CAM_DDS :
	STRUCT
		InstanceID : 				UDINT; 									(*Unique identifier for the FB instance. Is assigned at first activation*)
		FBStatus : 				iDD_INTERN_STATUS_DDS; 				(* Status value of state machine of FB*)
		ExecuteTrigR_0 : 			R_TRIG; 									(*Positive edge of Execute *)
		IntpDevice : UDINT;
		IntFileName : 				DD_MAX_STRING_DDS;
		FileHandle : 				iDD_INTERN_FILE_HANDLE_DDS;
		FileOpen_0 : 				iDD_FileOpen_DDS;
		FileRead_0 :				iDD_FileRead_DDS;
		FileClose_0 : 				iDD_FileClose_DDS;
		DDFileStatus : 			iDD_INTERN_FILETRANS_STAT_DDS;		(*Not completely correct, but can be reused here*)
		IntErrorId : 				WORD;
		EofReached : 			BOOL;									(*TRUE if the End of File is reached*)
		AbortAndRestart : 			BOOL;
		XML_READER_0:			iDD_XML_READER_DDS;
		myXML_CONTROL: 		iDD_INTERN_XML_CONTROL_DDS;
		ReadBuffer: 				ARRAY[0..iDD_MAX_BUFFER_INDEX] OF BYTE;
		DataPointCounter: 			UINT;
		ReadCounter : 			UINT;
		i : 						INT;
		Trim : 					iDD_TRIM_DDS;
		CamFileVersion : 			STRING[80] := '0.0.0.1';						(*This is actually constant.. but could not be declared as such, otherwise we get a warning during compiling*)
	END_STRUCT;
	iDD_INTERN_MOTION_CAM_DDS :
	STRUCT
		InstanceID : 					UDINT; 							(*Unique identifier for the FB instance. Is assigned at rising edge of Execute.*)
		PdoIdCounter : 				USINT;
		ExecuteTrigR_0 : 				R_TRIG; 							(* Positive edge of Execute *)
		MotTimer : 					USINT; 							(*Timer to count the cycles until the mode of operation changes*)
		AckTimer : 					USINT; 							(*Timer to count the cycles until the PDO counters match*)
		FBStatus : 					iDD_INTERN_STATUS_DDS; 		(* Status value of state machine of FB*)
		IntCamTableID : 				USINT;
		ActiveCamTableID_Old : 		USINT; 							(*CAM Table ID that was active in last cycle*)
		IntMasterOffset : 				DINT;
		IntSlaveOffset : 				REAL;
		IntChangeImmediate : 			BOOL;
		IntUseBlendDist : 				BOOL;
		IntBlendingDist : 				UDINT;
		IntSwitchControlSet : 			BOOL;
		EndOfProfileTrigR_0 : 			R_TRIG; 							(* Positive edge of EndOfProfile bit in Cam Profile status object*)
		FirstExecution : 				BOOL;
		IntBufFull : 					BOOL;
		IntBufCamTableID : 			USINT;
		IntBufMasterOffset : 			DINT;
		IntBufSlaveOffset : 				REAL;
		IntBufChangeImmediate : 		BOOL;
		IntBufUseBlendDist : 			BOOL;
		IntBufBlendingDist : 			UDINT;
		IntBufSwitchControlSet : 		BOOL;
		IntStartDelay : 				DD_CAM_DELAY_MODE_DDS;
	END_STRUCT;
	iDD_INTERN_MOTION_POS_DDS :
	STRUCT
		InstanceID : 				UDINT; 							(*Unique identifier for the FB instance. Is assigned at rising edge of Execute.*)
		PdoIdCounter : 			USINT;
		ExecuteTrigR_0 : 			R_TRIG;							(* Positive edge of Execute *)
		MotTimer : 				USINT; 							(*Timer to count the cycles until the mode of operation changes*)
		AckTimer : 				USINT; 							(*Timer to count the cycles until the PDO counters match*)
		FBStatus : 				iDD_INTERN_STATUS_DDS; 		(* Status value of state machine of FB*)
		IntPosition : 				DINT;
		IntVelocity : 				DINT;
		IntAcceleration : 			UDINT;
		IntDeceleration : 			UDINT;
		IntDirection : 				MC_DIRECTION_DDS;
		IntBufferMode : 			MC_BUFFER_MODE_DDS;
		IntTorqueLimit : 			UINT;
		FirstExecution : 			BOOL;
		IntBufFull : 				BOOL;
		IntBufPosition : 			DINT;
		IntBufVelocity : 			DINT;
		IntBufAcceleration : 		UDINT;
		IntBufDeceleration : 		UDINT;
		IntBufDirection : 			MC_DIRECTION_DDS;
		IntBufTorqueLimit : 		UINT;
	END_STRUCT;
	iDD_INTERN_PDO_EXC_DDS :
	STRUCT
		InstanceID : 			UDINT; 						(*Unique identifier for the FB instance. Is assigned at first activation*)
		PdoIdCounter : 		USINT;
		ExecuteTrigR_0 : 		R_TRIG;						(* Positive edge of Execute *)
		FBStatus : 			iDD_INTERN_STATUS_DDS;	(* Status value of state machine of FB*)
		MotTimer : 			USINT; 						(*Timer to count the cycles until the mode of operation changes*)
	END_STRUCT;
	iDD_INTERN_POWER_DDS :
	STRUCT
		InstanceID : 		UDINT; 						(*Unique identifier for the FB instance. Is assigned at first activation*)
		FBStatus : 		iDD_INTERN_STATUS_DDS; 	(* Status value of state machine of FB*)
		PowerOnTimer : 	iDD_TON_DDS;
	END_STRUCT;
	iDD_INTERN_PREFIX_FLAG_DDS :
	( (* Default: No prefix. Sign appears only for negative signed values *)
		iPREFIXFLAG_SIGN := 0, 		(* Prefix the output value with a sign (+ or ) if the output value is of a signed type. *)
		iPREFIXFLAG_BLANK, 		(* Prefix the output value with a blank if the output value is signed and positive *)
		iPREFIXFLAG_NULL, 			(* If width is prefixed with 0, zeros are added until the minimum width is reached. *)
		iPREFIXFLAG_HASH 			(* When used with the b, o, x, or X format, the # flag prefixes any nonzero output value with 2#, 8# or 16#. *)
		);
	iDD_INTERN_PREP_DCS_DDS :
	STRUCT
		InstanceID : 				UDINT;							(*Unique identifier for the FB instance. Is assigned at first activation*)
		FBStatus : 				iDD_INTERN_STATUS_DDS; 		(* Status value of state machine of FB*)
		FileTransfer_0 : 			iDD_FileTransfer_DDS; 			(*Transfers the CAM profile*)
		SdoWrite_0 : 				iDD_SdoWrite_DDS; 				(*Write the CAM configuration*)
		SdoRead_0 :				iDD_SdoRead_DDS;
		ExecuteTrigR_0 : 			R_TRIG; 							(* Positive edge of Execute *)
		IntpDevice : UDINT;
		IntFileName : 				STRING[80];
		DcsDownloadStatus : 		USINT;
		DcsConfiguration : 			UINT;
		ParsingState : 			WORD;
		ParsingErrorInfo : 			DWORD;
		TargetFileName : 			STRING[80];
	END_STRUCT;
	iDD_INTERN_READ_MOTION_DDS :
	STRUCT
		InstanceID : 		UDINT; 							(*Unique identifier for the FB instance. Is assigned at first activation*)
		FBStatus : 		iDD_INTERN_STATUS_DDS; 		(* Status value of state machine of FB*)
		Value : 			UINT;
		SdoRead_0 : 		iDD_SdoRead_DDS; 				(* Instance of SDO Reading function block*)
	END_STRUCT;
	iDD_INTERN_READ_VERS_DDS :
	STRUCT
		InstanceID : 			UDINT; 							(*Unique identifier for the FB instance. Is assigned at first activation*)
		SdoRead_0 : 			iDD_SdoRead_DDS; 				(* Instance of SDO Reading function block*)
		ExecuteTrigR_0 : 		R_TRIG; 							(* Positive edge of Execute *)
		ParamCounter :		USINT;
		FBStatus : 			iDD_INTERN_STATUS_DDS; 		(* Status value of state machine of FB*)
	END_STRUCT;
	iDD_INTERN_READ_VERS_SAB :
	STRUCT
		InstanceID : 		UDINT;						(*Unique identifier for the FB instance. Is assigned at first activation*)
		SdoRead_0 : 		iDD_SdoRead_SAB;			(* Instance of SDO Reading function block*)
		ExecuteTrigR_0 : 	R_TRIG; 						(* Positive edge of Execute *)
		ParamCounter : 	USINT;
		FBStatus : 		iDD_INTERN_STATUS_DDS;	(* Status value of state machine of FB*)
	END_STRUCT;
	iDD_INTERN_RESET_DDS :
	STRUCT
		InstanceID : 			UDINT; 							(*Unique identifier for the FB instance. Is assigned at first activation*)
		PdoIdCounter : 		USINT;
		ExecuteTrigR_0 : 		R_TRIG; 							(* Positive edge of Execute *)
		AckTimer : 			USINT; 							(*Timer to get Acknowledge*)
		FBStatus : 			iDD_INTERN_STATUS_DDS; 		(* Status value of state machine of FB*)
	END_STRUCT;
	iDD_INTERN_SAVE_CAM_DDS :
	STRUCT
		InstanceID : 				UDINT; 									(*Unique identifier for the FB instance. Is assigned at first activation*)
		FBStatus : 				iDD_INTERN_STATUS_DDS; 				(* Status value of state machine of FB*)
		ExecuteTrigR_0 : 			R_TRIG; 									(*Positive edge of Execute *)
		IntpDevice : UDINT;
		IntFileName : 				DD_MAX_STRING_DDS;
		FileHandle :				iDD_INTERN_FILE_HANDLE_DDS;
		FileDelete_0 :				iDD_FileDelete_DDS;
		FileOpen_0 : 				iDD_FileOpen_DDS;
		FileWrite_0 : 				iDD_FileWrite_DDS;
		FileClose_0 : 				iDD_FileClose_DDS;
		DDFileStatus :			iDD_INTERN_FILETRANS_STAT_DDS;		(*Not completely correct, but can be reused here*)
		IntErrorId : 				WORD;
		AbortAndRestart : 			BOOL;
		fbFormat: 				iDD_FormatString_DDS;
		sTemp : 					DD_MAX_STRING_DDS;
		DataPointCounter: 			UINT;
		WriteCounter : 			UINT;
		i : 						UINT;
		program_step :			INT;
	
		CamFileVersion : 			STRING[80] := '0.0.0.1';						(*This is actually constant.. but could not be declared as such, otherwise we get a warning during compiling*)
	END_STRUCT;
	iDD_INTERN_SDO_READ_DDS :
	STRUCT
		InstanceID : 		UDINT;						(*Unique identifier for the FB instance. Is assigned at rising edge of Execute.*)
		FBStatus : 		iDD_INTERN_STATUS_DDS; 	(*Status value of state machine of FB*)
		SdoRead_0 : 		iDD_SdoRead_DDS; 			(*Instance of SDO Reading function block*)
	END_STRUCT;
	iDD_INTERN_SDO_READ_PARAM_DDS :
	STRUCT
		InstanceID :			UDINT; 						(*Unique identifier for the FB instance. Is assigned at rising edge of Execute.*)
		ExecuteTrigR_0 : 		R_TRIG; 						(*Positive edge of Execute *)
		FBStatus : 			iDD_INTERN_STATUS_DDS;	(*Status value of state machine of FB*)
		SdoRead_0 : 			iDD_SdoRead_DDS; 			(*Instance of SDO reading function block*)
		Index : 				UINT; 						(*Index of the object to read*)
		Subindex : 			USINT; 						(*Subindex of the object to read*)
		Value: 				UDINT;
	END_STRUCT;
	iDD_INTERN_SDO_READ_SAB :
	STRUCT
		InstanceID : 		UDINT; 							(*Unique identifier for the FB instance. Is assigned at rising edge of Execute.*)
		FBStatus : 		iDD_INTERN_STATUS_DDS;		(*Status value of state machine of FB*)
		SdoRead_0 : 		iDD_SdoRead_SAB;				(*Instance of SDO Reading function block*)
	END_STRUCT;
	iDD_INTERN_SDO_WRITE_DDS :
	STRUCT
		InstanceID : 			UDINT;							(*Unique identifier for the FB instance. Is assigned at rising edge of Execute.*)
		ExecuteTrigR_0 : 		R_TRIG; 							(*Positive edge of Execute *)
		FBStatus : 			iDD_INTERN_STATUS_DDS; 		(*Status value of state machine of FB*)
		SdoWrite_0 : 			iDD_SdoWrite_DDS;				(* Instance of SDO Writing function block*)
	END_STRUCT;
	iDD_INTERN_SDO_WRITE_PARA_DDS :
	STRUCT
		InstanceID : 			UDINT; 							(*Unique identifier for the FB instance. Is assigned at rising edge of Execute.*)
		ExecuteTrigR_0 : 		R_TRIG; 							(*Positive edge of Execute *)
		FBStatus : 			iDD_INTERN_STATUS_DDS; 		(*Status value of state machine of FB*)
		SdoWrite_0 :			iDD_SdoWrite_DDS; 				(*Instance of SDO Writing function block*)
		Index : 				UINT; 							(*Index of the object to write*)
		Subindex : 			USINT; 							(*Subindex of the object to write*)
		Value : 				UDINT;
	END_STRUCT;
	iDD_INTERN_SDO_WRITE_SAB :
	STRUCT
		InstanceID : 			UDINT; 							(*Unique identifier for the FB instance. Is assigned at rising edge of Execute.*)
		ExecuteTrigR_0 : 		R_TRIG; 							(*Positive edge of Execute *)
		FBStatus :			iDD_INTERN_STATUS_DDS; 		(*Status value of state machine of FB*)
		SdoWrite_0 : 			iDD_SdoWrite_SAB;				(* Instance of SDO Writing function block*)
		Value : 				UDINT;
	END_STRUCT;
	iDD_INTERN_SEMAPHORE_DDS :
	(
		iDD_SemInit_DDS,
		iDD_SemAcquire_DDS,
		iDD_SemRelease_DDS
	);
	iDD_INTERN_STATUS_DDS :
	(
		ddStatus_Default_DDS,
		ddStatus_Start_DDS,
		ddStatus_Run_DDS,
		ddStatus_Abort_DDS,
		ddStatus_Done_DDS,
		ddStatus_Error_DDS,
		ddStatus_Restart_DDS
	);
	iDD_INTERN_TOUCH_DDS :
	STRUCT
		ExecuteTrigR_0 : 			R_TRIG; 								(* Positive edge of Execute *)
		InstanceID : 				UDINT; 								(*Unique identifier for the FB instance. Is assigned at first activation*)
		FBStatus : 				iDD_INTERN_STATUS_DDS; 			(* Status value of state machine of FB*)
		SdoRead_0 : 				iDD_SdoRead_DDS; 					(* Instance of SDO Reading function block*)
		SdoWrite_0 : 				iDD_SdoWrite_DDS; 					(* Instance of SDO Writing function block*)
		IntTriggerInput : 			UINT;
		IntWindowOnly : 			BOOL;
		IntFirstPosition : 			DINT;
		IntLastPosition : 			DINT;
		IntEdgeMode : 			DD_EDGE_MODE_DDS;
		UsedTriggerInput : 		UINT;
		Data : 					UINT; 								(*Data to be written to Touch probe function object*)
		DDTouchProbeStatus : 	iDD_INTERN_TOUCH_STATUS_DDS;
	END_STRUCT;
	iDD_INTERN_TOUCH_STATUS_DDS :
	(
		ddTouch_WriteDisable_DDS,
		ddTouch_WriteFirstPos_DDS,
		ddTouch_WriteLastPos_DDS,
		ddTouch_WriteEnable_DDS,
		ddTouch_WaitForTrigger_DDS,
		ddTouch_ReadTPStatus_DDS,
		ddTouch_ReadResult_DDS
		);
	iDD_INTERN_TRACE_DDS :
	STRUCT
		InstanceID : 				UDINT; 								(*Unique identifier for the FB instance. Is assigned at first activation*)
		SdoRead_0 :				iDD_SdoRead_DDS;					(* Instance of SDO Reading function block*)
		SdoWrite_0 : 				iDD_SdoWrite_DDS; 					(* Instance of SDO Writing function block*)
		ExecuteTrigR_0 : 			R_TRIG;								(* Positive edge of Execute *)
		FBStatus :				iDD_INTERN_STATUS_DDS; 			(* Status value of state machine of FB*)
		InternalTraceStatus : 		iDD_INTERN_TRACE_STATUS_DDS;
		WriteParameterState :		 INT;								(*Steps of reading and writing processes*)
		DataToSend : 			DWORD; 							(*Helper for conversion of data type*)
		MaxTraceBufSize : 		UDINT := 32000; 						(*The trace buffer size of the driv (given in samples: 1 sample = 4 Byte)*)
		ValidSignalIDNo : 			USINT;								(*Number of valid Signal IDs, valid when Signal ID > 0  *)
		SignalCounter : 			USINT; 								(*Counter of Signals to during writing *)
		TraceCtrlFlags : 			UINT; 								(*Information of object 0x5000, 01: Trace control flags*)
		PollDelay :				iDD_TON_DDS;						(*Delay time for polling the trace data ready information*)
		RestartAfterAbort : 			BOOL;
		IntpTraceBuffer : 			UDINT;
		IntTraceBufferSize : 		UDINT;
		IntSamplingRate : 			DD_SAMPLING_RATE_DDS;
		IntSampleCount : 			UINT;
		IntSubSampling : 			UINT;
		IntSignalIDs : 				ARRAY[0..7]OF UDINT;
		IntTriggerID :				UDINT;
		IntTriggerPoint : 			USINT;
		IntTriggerLevel : 			REAL;
		IntTriggerSlope : 			BOOL;
		i : 						USINT;
	END_STRUCT;
	iDD_INTERN_TRACE_SAB :
	STRUCT
		InstanceID : 				UDINT; 								(*Unique identifier for the FB instance. Is assigned at first activation*)
		SdoRead_0 : 				iDD_SdoRead_SAB;					(*Instance of SDO Reading function block*)
		SdoWrite_0 : 				iDD_SdoWrite_SAB; 					(*Instance of SDO Writing function block*)
		ExecuteTrigR_0 : 			R_TRIG;								(*Positive edge of Execute *)
		FBStatus : 				iDD_INTERN_STATUS_DDS; 			(*Status value of state machine of FB*)
		InternalTraceStatus : 		iDD_INTERN_TRACE_STATUS_DDS;
		WriteParameterState :		INT; 								(*Steps of reading and writing processes*)
		DataToSend : 			DWORD;							(*Helper for conversion of data type*)
		MaxTraceBufSize : 		UINT := 32000; 						(*The trace buffer size of the drive (given in samples; 1 sample = 4 Byte)*)
		ValidSignalIDNo : 			USINT; 								(*Number of valid Signal IDs, valid when Signal ID > 0  *)
		SignalCounter : 			USINT; 								(*Counter of Signals to during writing *)
		TraceCtrlFlags : 			UINT; 								(*Information of object 0x5000, 01: Trace control flags*)
		PollDelay : 				iDD_TON_DDS; 						(*Delay time for polling the trace data ready information*)
		RestartAfterAbort : 			BOOL;
		IntpTraceBuffer : 			UDINT;
		IntTraceBufferSize : 		UDINT;
		IntSamplingRate : 			DD_SAMPLING_RATE_DDS;
		IntSampleCount : 			UINT;
		IntSubSampling : 			UINT;
		IntSignalIDs : 				ARRAY[0..7]OF UDINT;
		IntTriggerID : 				UDINT;
		IntTriggerPoint : 			USINT;
		IntTriggerLevel : 			REAL;
		IntTriggerSlope : 			BOOL;
		NewParameter : 			BOOL;
		i : 						USINT;
	END_STRUCT;
	iDD_INTERN_TRACE_STATUS_DDS :
	(
		ddTrace_Default_DDS,
		ddTrace_WaitBeforeRead_DDS,
		ddTrace_ReadData_DDS,
		ddTrace_WaitBeforeUpload_DDS,
		ddTrace_UploadData_DDS,
		ddTrace_CmdAbort_DDS,
		ddTrace_Aborting_DDS
	);
	iDD_INTERN_TRIGGER_DDS :
	STRUCT
		InstanceID : 		UDINT; 						(*Unique identifier for the FB instance. Is assigned at first activation*)
		FBStatus :		iDD_INTERN_STATUS_DDS; 	(* Status value of state machine of FB*)
		FallingEdge :		F_TRIG;
		RisingEdge :		R_TRIG;
	END_STRUCT;
	iDD_INTERN_TYPEFIELDPARA_DDS :
	(
		iTYPEFIELD_UNKNOWN := 0,
		iTYPEFIELD_B, 				(* b or B: binary number *)
		iTYPEFIELD_O,				(* o or O: octal number *)
		iTYPEFIELD_U, 				(* u or U: unsigned decimal number *)
		iTYPEFIELD_C, 				(* c or C: one ASCII character *)
		iTYPEFIELD_F, 				(* f or F: float number ( normalized format )*)
		iTYPEFIELD_D, 				(* d or D: signed decimal number  *)
		iTYPEFIELD_S,				(* s or S: string *)
		iTYPEFIELD_XU, 				(* X: hecadecimal number (upper case characters )*)
		iTYPEFIELD_XL, 				(* x: hecadecimal number (lower case characters )*)
		iTYPEFIELD_EU,				(* E: float number ( scientific format ) *)
		iTYPEFIELD_EL 				(* e: float number ( scientific format ) *)
	);
	iDD_INTERN_UPDATE_FW_DDS :
	STRUCT
		InstanceID : 			UDINT; 						(*Unique identifier for the FB instance. Is assigned at first activation*)
		FBStatus : 			iDD_INTERN_STATUS_DDS; 	(* Status value of state machine of FB*)
		FileTransfer_0 : 		iDD_FileTransfer_DDS; 		(*Transfers the CAM profile*)
		ExecuteTrigR_0 : 		R_TRIG; 						(* Positive edge of Execute *)
		IntpDevice : UDINT;
		IntFileName : 			STRING[80];
		TargetFileName : 		STRING[80] := 'ISD500.bin'; 		(*This also includes the path. In case of firmware download, there is no path.*)
	END_STRUCT;
	iDD_INTERN_WRITE_SCALING_DDS :
	STRUCT
		InstanceID : 					UDINT; 							(*Unique identifier for the FB instance. Is assigned at first activation*)
		SdoWrite_0 :					iDD_SdoWrite_DDS; 				(* Instance of SDO Writing function block*)
		ExecuteTrigR_0 : 				R_TRIG;							(* Positive edge of Execute *)
		ParamCounter : 				USINT;
		IntMasterScalingNumerator : 	UDINT;
		IntMasterScalingDenominator : 	UDINT;
		IntSlaveScalingNumerator : 		UDINT;
		IntSlaveScalingDenominator : 	UDINT;
		FBStatus : 					iDD_INTERN_STATUS_DDS; 		(* Status value of state machine of FB*)
	END_STRUCT;
	iDD_INTERN_XML_CONTROL_DDS :
	STRUCT
		COMMAND : 			WORD;
		WATCHDOG : 		TIME;
		START_POS : 		UINT;
		STOP_POS : 			UINT;
		COUNT : 			UINT;
		TYP : 				INT;
		LEVEL : 				UINT;
		PATH : 				DD_MAX_STRING_DDS;
		ELEMENT : 			DD_MAX_STRING_DDS;
		ATTRIBUTE : 		DD_MAX_STRING_DDS;
		VALUE : 				DD_MAX_STRING_DDS;
		BLOCK1_START :		UINT;
		BLOCK1_STOP : 		UINT;
		BLOCK2_START : 	UINT;
		BLOCK2_STOP : 		UINT;
	END_STRUCT;
	iDD_SAB_STATE_SAB :
	( (*States of the SAB state machine*)
		sab_Unknown_SAB, 			(*State is unknown*)
		sab_Init_SAB,
		sab_UauxDisabled_SAB,
		sab_Standby_SAB,
		sab_PowerUp_SAB,
		sab_OperationEnabled_SAB,
		sab_Fault_SAB
	);
	iDD_SAB_REF_INTERN_SAB :
	STRUCT
		OperatingState : 			iDD_SAB_STATE_SAB := sab_Unknown_SAB;
		iDD_CycleInfo_DDS_0 : 	iDD_CycleInfo_DDS; 									(*Information of times of calling the current cylce "CycleCounter"*)
		NewCommand : 			BOOL; 												(*Command to send (RPDO) to drive/SAB on FB request *) (*PDO administrative variables*)
		FosSemaphore : 			iDD_Semaphore_DDS; 								(*Initialization of the FoS semaphore *)
		InitDone : 				BOOL; 												(*One time initialization of SAB structure *)
		AdrCounter : 				USINT; 												(*Current postion index in the TPDO array for unpacking resp. RPDO array for packing the variables*)
		ContextSelector : 			USINT := iDD_CONTEXT_SEL_SAB_ENCODER_SIM; 		(*Identifier of context selector for RPDO*)
		Controlword : 				UINT; 												(*RPDO-References 0x4040*)
		PdoRxId : 				USINT; 												(*Index of RPDO: Used for handshaking of commands*)
		PdoRxIdCs : 				USINT; 												(*RPDO contains 2 Bits for RPDO-ID and 6 Bits for Context Selector*) (*Single parameters for RPDO; are packed into the RPDO, depending on the context selector*)
		EncoderControl : 			UINT;
		EncoderSimVel : 			REAL;
		EncoderSimAcc : 			REAL;
		EncoderSimDec : 			REAL;
		EncoderSimRampJerk : 	UDINT;
		Statusword : 				UINT; 												(*TPDO-References 0x4041*)
		DcLinkVoltage : 			INT;													(*TPDO-References 0x2001.04*)
		DcLinkPower : 			REAL;
		PosGuideValueRef : 		UDINT;
		VelGuideValueRef : 		REAL;
		PdoTxId : 				USINT; 												(*TPDO-References 0x5041*)
	END_STRUCT;
	iDD_MOTIONSTATUS_DDS :
	(	(*States of the PLCopen state machine*)
		mcUnknown_DDS,
		mcDisabled_DDS,
		mcStandstill_DDS,
		mcHoming_DDS,
		mcStopping_DDS,
		mcContinuousMotion_DDS,
		mcDiscreteMotion_DDS,
		mcSynchronizedMotion_DDS,
		mcErrorstop_DDS
	);
	iDD_ACM_REF_INTERN_ACM :
	STRUCT
		OperatingState : 			iDD_ACM_STATE_ACM := acm_Unknown_ACM;
		ControlMode :				DD_CONTROL_MODE_DDS; 							(* Indicates the current control mode *)
		Remote:					BOOL;												(* Fieldbus mode enable *)
		iDD_CycleInfo_DDS_0 : 	iDD_CycleInfo_DDS; 									(*Information of times of calling the current cylce "CycleCounter"*)
		NewCommand : 			BOOL; 												(*Command to send (RPDO) to drive/ACM on FB request *) (*PDO administrative variables*)
		FosSemaphore : 			iDD_Semaphore_DDS; 								(*Initialization of the FoS semaphore *)
		InitDone : 				BOOL; 												(*One time initialization of ACM structure *)
		AdrCounter : 				USINT; 												(*Current postion index in the TPDO array for unpacking resp. RPDO array for packing the variables*)
		ContextSelector : 			USINT :=iDD_CONTEXT_SEL_ACM_DISABLED;	 		(*Identifier of context selector for RPDO*)
		Controlword : 				UINT; 												(*RPDO-References 0x4040*)
		PdoRxId : 				USINT; 												(*Index of RPDO: Used for handshaking of commands*)
		PdoRxIdCs : 				USINT; 												(*RPDO contains 2 Bits for RPDO-ID and 6 Bits for Context Selector*) (*Single parameters for RPDO; are packed into the RPDO, depending on the context selector*)
		Statusword : 				UINT; 												(*TPDO-References 0x4041*)
		DcLinkVoltage : 			INT;													(*TPDO-References 0x2001.04*)
		Reserved_1: 				UDINT;
		Reserved_2: 				UDINT;
		Reserved_3: 				UDINT;
		DigitalOutputStatus :		UINT;												(* Status of digital outputs *)
		PdoTxId : 				USINT; 												(*TPDO-References 0x5041*)
	END_STRUCT;
	iDD_ACM_STATE_ACM :
	(
		(*States of the ACM state machine*)
		acm_Unknown_ACM, 			(*State is unknown*)
		acm_Init_ACM,
		acm_NotReady_ACM,
		acm_Standby_ACM,
		acm_PowerUp_ACM,
		acm_OperationEnabled_ACM,
		acm_FaultReaction_ACM,
		acm_Fault_ACM
	
	);
	iDD_INTERN_READ_VERS_ACM :
	STRUCT
		InstanceID : 		UDINT;						(*Unique identifier for the FB instance. Is assigned at first activation*)
		SdoRead_0 : 		iDD_SdoRead_ACM;			(* Instance of SDO Reading function block*)
		ExecuteTrigR_0 : 	R_TRIG; 						(* Positive edge of Execute *)
		ParamCounter : 	USINT;
		FBStatus : 		iDD_INTERN_STATUS_DDS;	(* Status value of state machine of FB*)
	END_STRUCT;
	iDD_INTERN_SDO_READ_ACM :
	STRUCT
		InstanceID : 		UDINT; 							(*Unique identifier for the FB instance. Is assigned at rising edge of Execute.*)
		FBStatus : 		iDD_INTERN_STATUS_DDS;		(*Status value of state machine of FB*)
		SdoRead_0 : 		iDD_SdoRead_ACM;				(*Instance of SDO Reading function block*)
	END_STRUCT;
	iDD_INTERN_SDO_WRITE_ACM :
	STRUCT
		InstanceID : 			UDINT; 							(*Unique identifier for the FB instance. Is assigned at rising edge of Execute.*)
		ExecuteTrigR_0 : 		R_TRIG; 							(*Positive edge of Execute *)
		FBStatus :			iDD_INTERN_STATUS_DDS; 		(*Status value of state machine of FB*)
		SdoWrite_0 : 			iDD_SdoWrite_ACM;				(* Instance of SDO Writing function block*)
		Value : 				UDINT;
	END_STRUCT;
	iDD_INTERN_TRACE_ACM :
	STRUCT
		InstanceID : 				UDINT; 								(*Unique identifier for the FB instance. Is assigned at first activation*)
		SdoRead_0 : 				iDD_SdoRead_ACM;					(*Instance of SDO Reading function block*)
		SdoWrite_0 : 				iDD_SdoWrite_ACM; 					(*Instance of SDO Writing function block*)
		ExecuteTrigR_0 : 			R_TRIG;								(*Positive edge of Execute *)
		FBStatus : 				iDD_INTERN_STATUS_DDS; 			(*Status value of state machine of FB*)
		InternalTraceStatus : 		iDD_INTERN_TRACE_STATUS_DDS;
		WriteParameterState :		INT; 								(*Steps of reading and writing processes*)
		DataToSend : 			DWORD;							(*Helper for conversion of data type*)
		MaxTraceBufSize : 		UINT := 32000; 						(*The trace buffer size of the drive (given in samples; 1 sample = 4 Byte)*)
		ValidSignalIDNo : 			USINT; 								(*Number of valid Signal IDs, valid when Signal ID > 0  *)
		SignalCounter : 			USINT; 								(*Counter of Signals to during writing *)
		TraceCtrlFlags : 			UINT; 								(*Information of object 0x5000, 01: Trace control flags*)
		PollDelay : 				iDD_TON_DDS; 						(*Delay time for polling the trace data ready information*)
		RestartAfterAbort : 			BOOL;
		IntpTraceBuffer : 			UDINT;
		IntTraceBufferSize : 		UDINT;
		IntSamplingRate : 			DD_SAMPLING_RATE_DDS;
		IntSampleCount : 			UINT;
		IntSubSampling : 			UINT;
		IntSignalIDs : 				ARRAY[0..7]OF UDINT;
		IntTriggerID : 				UDINT;
		IntTriggerPoint : 			USINT;
		IntTriggerLevel : 			REAL;
		IntTriggerSlope : 			BOOL;
		NewParameter : 			BOOL;
		i : 						USINT;
	END_STRUCT;
	iDD_DAM_REF_INTERN_DAM :
	STRUCT
		OperatingState : 			iDD_DAM_STATE_DAM := dam_Unknown_DAM;
		ControlMode :				DD_CONTROL_MODE_DDS; 							(* Indicates the current control mode *)
		Remote:					BOOL;												(* Fieldbus mode enable *)
		iDD_CycleInfo_DDS_0 : 	iDD_CycleInfo_DDS; 									(*Information of times of calling the current cylce "CycleCounter"*)
		NewCommand : 			BOOL; 												(*Command to send (RPDO) to drive/SAB on FB request *) (*PDO administrative variables*)
		FosSemaphore : 			iDD_Semaphore_DDS; 								(*Initialization of the FoS semaphore *)
		InitDone : 				BOOL; 												(*One time initialization of SAB structure *)
		AdrCounter : 				USINT; 												(*Current postion index in the TPDO array for unpacking resp. RPDO array for packing the variables*)
		ContextSelector : 			USINT := iDD_CONTEXT_SEL_DAM_ENCODER_SIM; 		(*Identifier of context selector for RPDO*)
		Controlword : 				UINT; 												(*RPDO-References 0x4040*)
		PdoRxId : 				USINT; 												(*Index of RPDO: Used for handshaking of commands*)
		PdoRxIdCs : 				USINT; 												(*RPDO contains 2 Bits for RPDO-ID and 6 Bits for Context Selector*) (*Single parameters for RPDO; are packed into the RPDO, depending on the context selector*)
		EncoderControl : 			UINT;
		EncoderSimVel : 			REAL;
		EncoderSimAcc : 			REAL;
		EncoderSimDec : 			REAL;
		EncoderSimRampJerk : 	UDINT;
		Statusword : 				UINT; 												(*TPDO-References 0x4041*)
		DcLinkVoltage : 			INT;													(*TPDO-References 0x2001.04*)
		DcLinkPower : 			REAL;
		PosGuideValueRef : 		UDINT;
		VelGuideValueRef : 		REAL;
		PdoTxId : 				USINT; 												(*TPDO-References 0x5041*)
	END_STRUCT;
	iDD_DAM_STATE_DAM :
	(
		 (*States of the DAM state machine*)
	
		dam_Unknown_DAM, 			(*State is unknown*)
		dam_Init_DAM,
		dam_NotReady_DAM,
		dam_Standby_DAM,
		dam_PowerUp_DAM,
		dam_OperationEnabled_DAM,
		dam_FaultReaction_DAM,
		dam_Fault_DAM
	
	
	);
	iDD_INTERN_READ_VERS_DAM :
	STRUCT
		InstanceID : 		UDINT;						(*Unique identifier for the FB instance. Is assigned at first activation*)
		SdoRead_0 : 		iDD_SdoRead_DAM;			(* Instance of SDO Reading function block*)
		ExecuteTrigR_0 : 	R_TRIG; 						(* Positive edge of Execute *)
		ParamCounter : 	USINT;
		FBStatus : 		iDD_INTERN_STATUS_DDS;	(* Status value of state machine of FB*)
	END_STRUCT;
	iDD_INTERN_SDO_READ_DAM :
	STRUCT
		InstanceID : 		UDINT; 							(*Unique identifier for the FB instance. Is assigned at rising edge of Execute.*)
		FBStatus : 		iDD_INTERN_STATUS_DDS;		(*Status value of state machine of FB*)
		SdoRead_0 : 		iDD_SdoRead_DAM;				(*Instance of SDO Reading function block*)
	END_STRUCT;
	iDD_INTERN_SDO_WRITE_DAM :
	STRUCT
		InstanceID : 			UDINT; 							(*Unique identifier for the FB instance. Is assigned at rising edge of Execute.*)
		ExecuteTrigR_0 : 		R_TRIG; 							(*Positive edge of Execute *)
		FBStatus :			iDD_INTERN_STATUS_DDS; 		(*Status value of state machine of FB*)
		SdoWrite_0 : 			iDD_SdoWrite_DAM;				(* Instance of SDO Writing function block*)
		Value : 				UDINT;
	END_STRUCT;
	iDD_INTERN_TRACE_DAM :
	STRUCT
		InstanceID : 				UDINT; 								(*Unique identifier for the FB instance. Is assigned at first activation*)
		SdoRead_0 : 				iDD_SdoRead_DAM;					(*Instance of SDO Reading function block*)
		SdoWrite_0 : 				iDD_SdoWrite_DAM; 					(*Instance of SDO Writing function block*)
		ExecuteTrigR_0 : 			R_TRIG;								(*Positive edge of Execute *)
		FBStatus : 				iDD_INTERN_STATUS_DDS; 			(*Status value of state machine of FB*)
		InternalTraceStatus : 		iDD_INTERN_TRACE_STATUS_DDS;
		WriteParameterState :		INT; 								(*Steps of reading and writing processes*)
		DataToSend : 			DWORD;							(*Helper for conversion of data type*)
		MaxTraceBufSize : 		UINT := 32000; 						(*The trace buffer size of the drive (given in samples; 1 sample = 4 Byte)*)
		ValidSignalIDNo : 			USINT; 								(*Number of valid Signal IDs, valid when Signal ID > 0  *)
		SignalCounter : 			USINT; 								(*Counter of Signals to during writing *)
		TraceCtrlFlags : 			UINT; 								(*Information of object 0x5000, 01: Trace control flags*)
		PollDelay : 				iDD_TON_DDS; 						(*Delay time for polling the trace data ready information*)
		RestartAfterAbort : 			BOOL;
		IntpTraceBuffer : 			UDINT;
		IntTraceBufferSize : 		UDINT;
		IntSamplingRate : 			DD_SAMPLING_RATE_DDS;
		IntSampleCount : 			UINT;
		IntSubSampling : 			UINT;
		IntSignalIDs : 				ARRAY[0..7]OF UDINT;
		IntTriggerID : 				UDINT;
		IntTriggerPoint : 			USINT;
		IntTriggerLevel : 			REAL;
		IntTriggerSlope : 			BOOL;
		NewParameter : 			BOOL;
		i : 						USINT;
	END_STRUCT;
	iDD_INTERN_READ_VERS_PSM :
	STRUCT
		InstanceID : 		UDINT;						(*Unique identifier for the FB instance. Is assigned at first activation*)
		SdoRead_0 : 		iDD_SdoRead_PSM;			(* Instance of SDO Reading function block*)
		ExecuteTrigR_0 : 	R_TRIG; 						(* Positive edge of Execute *)
		ParamCounter : 	USINT;
		FBStatus : 		iDD_INTERN_STATUS_DDS;	(* Status value of state machine of FB*)
	END_STRUCT;
	iDD_INTERN_SDO_READ_PSM :
	STRUCT
		InstanceID : 		UDINT; 							(*Unique identifier for the FB instance. Is assigned at rising edge of Execute.*)
		FBStatus : 		iDD_INTERN_STATUS_DDS;		(*Status value of state machine of FB*)
		SdoRead_0 : 		iDD_SdoRead_PSM;				(*Instance of SDO Reading function block*)
	END_STRUCT;
	iDD_INTERN_SDO_WRITE_PSM :
	STRUCT
		InstanceID : 			UDINT; 							(*Unique identifier for the FB instance. Is assigned at rising edge of Execute.*)
		ExecuteTrigR_0 : 		R_TRIG; 							(*Positive edge of Execute *)
		FBStatus :			iDD_INTERN_STATUS_DDS; 		(*Status value of state machine of FB*)
		SdoWrite_0 : 			iDD_SdoWrite_PSM;				(* Instance of SDO Writing function block*)
		Value : 				UDINT;
	END_STRUCT;
	iDD_INTERN_TRACE_PSM :
	STRUCT
		InstanceID : 				UDINT; 								(*Unique identifier for the FB instance. Is assigned at first activation*)
		SdoRead_0 : 				iDD_SdoRead_PSM;					(*Instance of SDO Reading function block*)
		SdoWrite_0 : 				iDD_SdoWrite_PSM; 					(*Instance of SDO Writing function block*)
		ExecuteTrigR_0 : 			R_TRIG;								(*Positive edge of Execute *)
		FBStatus : 				iDD_INTERN_STATUS_DDS; 			(*Status value of state machine of FB*)
		InternalTraceStatus : 		iDD_INTERN_TRACE_STATUS_DDS;
		WriteParameterState :		INT; 								(*Steps of reading and writing processes*)
		DataToSend : 			DWORD;							(*Helper for conversion of data type*)
		MaxTraceBufSize : 		UINT := 32000; 						(*The trace buffer size of the drive (given in samples; 1 sample = 4 Byte)*)
		ValidSignalIDNo : 			USINT; 								(*Number of valid Signal IDs, valid when Signal ID > 0  *)
		SignalCounter : 			USINT; 								(*Counter of Signals to during writing *)
		TraceCtrlFlags : 			UINT; 								(*Information of object 0x5000, 01: Trace control flags*)
		PollDelay : 				iDD_TON_DDS; 						(*Delay time for polling the trace data ready information*)
		RestartAfterAbort : 			BOOL;
		IntpTraceBuffer : 			UDINT;
		IntTraceBufferSize : 		UDINT;
		IntSamplingRate : 			DD_SAMPLING_RATE_DDS;
		IntSampleCount : 			UINT;
		IntSubSampling : 			UINT;
		IntSignalIDs : 				ARRAY[0..7]OF UDINT;
		IntTriggerID : 				UDINT;
		IntTriggerPoint : 			USINT;
		IntTriggerLevel : 			REAL;
		IntTriggerSlope : 			BOOL;
		NewParameter : 			BOOL;
		i : 						USINT;
	END_STRUCT;
	iDD_PSM_REF_INTERN_PSM :
	STRUCT
		OperatingState : 			iDD_PSM_STATE_PSM := psm_Unknown_PSM;
		ControlMode :				DD_CONTROL_MODE_DDS; 							(* Indicates the current control mode *)
		Remote:					BOOL;												(* Fieldbus mode enable *)
		iDD_CycleInfo_DDS_0 : 	iDD_CycleInfo_DDS; 									(*Information of times of calling the current cylce "CycleCounter"*)
		NewCommand : 			BOOL; 												(*Command to send (RPDO) to drive/PSM on FB request *) (*PDO administrative variables*)
		FosSemaphore : 			iDD_Semaphore_DDS; 								(*Initialization of the FoS semaphore *)
		InitDone : 				BOOL; 												(*One time initialization of PSM structure *)
		AdrCounter : 				USINT; 												(*Current postion index in the TPDO array for unpacking resp. RPDO array for packing the variables*)
		ContextSelector : 			USINT := iDD_CONTEXT_SEL_PSM_DISABLED;	 		(*Identifier of context selector for RPDO*)
		Controlword : 				UINT; 												(*RPDO-References 0x4040*)
		PdoRxId : 				USINT; 												(*Index of RPDO: Used for handshaking of commands*)
		PdoRxIdCs : 				USINT; 												(*RPDO contains 2 Bits for RPDO-ID and 6 Bits for Context Selector*) (*Single parameters for RPDO; are packed into the RPDO, depending on the context selector*)
		Statusword : 				UINT; 												(*TPDO-References 0x4041*)
		DcLinkVoltage : 			INT;													(*TPDO-References 0x2001.04*)
		DcLinkPower : 			REAL;
		Reserved_1: 				UDINT;
		Reserved_2 : 				UDINT;
		DigitalOutputStatus :		UINT;												(* Status of digital outputs *)
		PdoTxId : 				USINT; 												(*TPDO-References 0x5041*)
	END_STRUCT;
	iDD_PSM_STATE_PSM :
	(
		(*States of the PSM state machine*)
	
		psm_Unknown_PSM, 			(*State is unknown*)
		psm_Init_PSM,
		psm_NotReady_PSM,
		psm_Standby_PSM,
		psm_PowerUp_PSM,
		psm_OperationEnabled_PSM,
		psm_FaultReaction_PSM,
		psm_Fault_PSM
	
	
	);
	iDD_INTERN_WRITE_CONFIG_PARA_DDS :
	STRUCT
		InstanceID : 			UDINT; 									(*Unique identifier for the FB instance. Is assigned at first activation*)
		FBStatus : 			iDD_INTERN_STATUS_DDS; 				(* Status value of state machine of FB*)
		FileTransfer_0 : 		iDD_FileTransfer_DDS; 					(*Transfers the CAM profile*)
		ExecuteTrigR_0 : 		R_TRIG; 									(* Positive edge of Execute *)
		IntpDevice : UDINT;								(*The variable of this type selects generic or one of the TwinCAT system paths on the target device*)
		IntFileName : 			STRING[80];								(*Filename of the config parameter file on the PLC*)
		TargetFileName : 		STRING[80] := 'DSP/fs/ConfigParam.dat'; 		(*This also includes the path. In case of firmware download, there is no path.*)
	END_STRUCT;
	iDD_INTERN_RES_AMPL_CALIBR_DDS :
		STRUCT
			InstanceID : 				UDINT; 									(*Unique identifier for the FB instance. Is assigned at first activation*)
			SdoRead_0 : 				iDD_SdoRead_DDS; 						(*Instance of SDO Reading function block*)
			ExecuteTrigR_0 : 			R_TRIG; 									(*Positive edge of Execute *)
			FBStatus : 				iDD_INTERN_STATUS_DDS; 				(*Status value of state machine of FB*)
			DDResolverStatus : 		iDD_INTERN_RESOLVER_STATUS_DDS; 	(*Internal state machine variable*)
			ResolverCalibrationStatus : 	UINT;
			SdoWrite_0 :				iDD_SdoWrite_DDS; 						(* Instance of SDO Writing function block*)
			PdoIdCounter : 			USINT;
			IntTargetVelocity : 			DINT; 									(*Value of the target velocity [u/s]. Can be a signed value.*)
			IntAcceleration : 			UDINT; 									(*Value of the "Acceleration" (always positiive) (increasing energy of the motor) [u/s]*)
			IntDeceleration : 			UDINT; 									(*Value of the "Deceleration" (always positive) (decreasing energy of the motor) [u/s]*)
			MotTimer : 				USINT; 									(*Timer to count the cycles until the mode of operation changes*)
		END_STRUCT;
	iDD_INTERN_RESOLVER_STATUS_DDS :
	(
		ddResolver_Default_DDS,
		ddResolver_WriteTargetVelocityE0,
		ddResolver_WriteAcceleration_DDS,
		ddResolver_WriteDeceleration_DDS,
		ddResolver_StartCalibration_DDS,
		ddResolver_Evaluation_DDS,
		ddResolver_FinishCalibration_DDS
	);
END_TYPE
