
TYPE
	DD_AbsolutePositioning_DDS :
	STRUCT
		StartPlateAngle :	REAL := 0;	(*Absolute plate angle at the beginning of the block.*)
		EndPlateAngle :	REAL := 0;	(*Absolute plate angle at the end of the block.*)
		StartVel :			REAL := 0;	(*Velocity of the axis at the beginning of this block. The velocity has to be given as a factor between the velocity of the axis in relation to the velocity of the guide value.*)
		EndVel : 			REAL := 0;	(*Same as StartVel but at the end of the block.*)
		StartAcc :		REAL := 0;	(*Acceleration of the axis at the beginning of this block. The acceleration has to be given as a factor between the acceleration of the axis in relation to the acceleration of the guide value.*)
		EndAcc : 		REAL := 0;	(*Same as StartAcc but at the end of the block.*)
	END_STRUCT;
	DD_ConstantGear_DDS :
	STRUCT
		AccGuideAngle : 		REAL := 0;		(*Relative angle of the guide value which is used to accelerate from standstill to the desired gear ratio (defined by the ratio of ConstPlateAngle per ConstGuideAngle). *)
		ConstPlateAngle : 		REAL := 0;		(*Relative plate angle that should be turned at the constant gear ratio. Note that a negative angle leads to a movement in backward direction. Given in degree.*)
		ConstGuideAngle : 	REAL := 0;		(*Relative angle of the guide value that is used for the constant gear ratio part..*)
	END_STRUCT;
	DD_ConstantSpeed_DDS :
	STRUCT
		AccTime : 			UINT := 5;	(*Relative time (duration) which is used to accelerate from standstill to the desired speed. Given in ms.*)
		ConstPlateAngle : 		REAL := 0;	(*Relative plate angle that should be turned at the constant speed part. Note that a negative angle leads to a movement in backward direction. Given in degree.*)
		ConstPlateSpeed :	REAL := 0;	(*Speed given in rps that is used for the constant speed part. The speed is seen as an absolute value (use positive speed also for backward movement).*)
		DecTime : 			UINT := 5;	(*Relative time (duration) which is used to decelerate from the given speed to standstill. Given in ms.*)
	END_STRUCT;
	DD_DynamicAlignment_DDS :
	STRUCT
		StartVel :		REAL := 0;	(*Velocity of the axis at the beginning of this block. The velocity has to be given as a factor between the velocity of the axis in relation to the velocity of the guide value.*)
		EndVel : 		REAL := 0;	(*Same as StartVel but at the end of the block.*)
		StartAcc :	REAL := 0;	(*Acceleration of the axis at the beginning of this block. The acceleration has to be given as a factor between the acceleration of the axis in relation to the acceleration of the guide value.*)
		EndAcc : 	REAL := 0;	(*Same as StartAcc but at the end of the block.*)
		SegmentID :	UINT := 0;	(*Segment ID that should be assigned to the underlying segment. This ID is required for sending the parameter to this segment during run-time. (e.g. the angle of the camera has to be sent to the ID given here).
								This segment ID needs to be unique for this CAM.*)
	END_STRUCT;
	DD_Init_DDS :
	STRUCT
		EndPlateAngle: 	REAL := 0;	(*Desired end rotor position relative to the nearest physical position.
									 The reference-position is determined by the absolute position at the beginning of this block 
									 and the partition / revolutions. Given in revolutions.*)
		Partition : 		USINT := 0;	(*Can be used for shaped plates, when several equal, valid starting positions are allowed. Note that the worst case movement is influenced by this parameter.*)
		Revolutions : 		UINT:= 1;		(*Number of revolutions that are used when calculating valid positions, e.g. if there is a gear.*)
	END_STRUCT;
	DD_LABEL_BLOCK_DDS :
	STRUCT
		StartGuideValue: 		REAL := 0;													(*Guide value at which this CAM block starts.
																						Given in revolutions of guide value.
																						Value range: Between 0 and 1 (1 not included)*)
		DeltaGuideValue: 		REAL := 0;													(*Length of the block. End of block is StartGuideValue + DeltaGuideValue.*)
		BlockType: 			DD_LabelBlockTypes_DDS := DD_Labeling_Unknown_DDS;		(*Specifies the type of label block*)
		Init: 					DD_Init_DDS;													(*Parameters related to re-initialize label block*)
		RelPos: 				DD_RelativePositioning_DDS;									(*Parameters related to relative positioning label block*)
		AbsPos: 				DD_AbsolutePositioning_DDS;									(*Parameters related to absolute positioning label block*)
		ConstGear: 			DD_ConstantGear_DDS;										(*Parameters related to constant gear label block*)
		ConstSpeed: 			DD_ConstantSpeed_DDS;										(*Parameters related to constant speed label block*)
		StaticAlign: 			DD_StaticAlignment_DDS;										(*Parameters related to static alignment label block*)
		DynAlign: 			DD_DynamicAlignment_DDS;									(*Parameters related to dynamic alignment label block*)
		MarkAlign: 			DD_MarkAlignment_DDS;										(*Parameters related to mark alignment label block*)
		PatternAlign: 			DD_PatternAlignment_DDS;										(*Parameters related to pattern alignment label block*)
	END_STRUCT;
	DD_LABEL_CAM_DDS :
	STRUCT
		MasterScaling : 			DD_FACTOR_DDS;													(*Parameters for master scaling*)
		SlaveScaling : 			DD_FACTOR_DDS; 													(*Parameters for slave scaling*)
		ControlParameterSet1 : 	DD_CONTROL_PARAMETER_DDS;										(*Parameters for control parameter set 1*)
		ControlParameterSet2 : 	DD_CONTROL_PARAMETER_DDS; 									(*Parameters for control parameter set 2*)
		FollowingError : 			DD_FOLLOWING_ERROR_DDS; 										(*Parameters for following error set-up*)
		LabelBlocks : 			ARRAY[1..DD_MAX_LABEL_BLOCKS_DDS]OF DD_LABEL_BLOCK_DDS; 		(*Array of label blocks for the Labeling CAM*)
		NumberOfBlocks : 			UINT := 0; 															(*Gives the number of used label blocks inside the array*)
	END_STRUCT;
	DD_LabelBlockTypes_DDS :
	(
		DD_Labeling_Unknown_DDS,
		DD_Labeling_Init_DDS,
		DD_Labeling_Standstill_DDS,
		DD_Labeling_RelPos_DDS,
		DD_Labeling_AbsPos_DDS,
		DD_Labeling_ConstGear_DDS,
		DD_Labeling_ConstSpeed_DDS,
		DD_Labeling_StaticAlign_DDS,
		DD_Labeling_DynAlign_DDS,
		DD_Labeling_MarkAlign_DDS,
		DD_Labeling_PatternAlign_DDS
	);
	DD_MarkAlignment_DDS :
	STRUCT
		AccTime : 		UINT 	:= 5;		(*Relative time (duration) which is used to accelerate from standstill to the desired speed. Given in ms.*)
		ConstPlateAngle : 	REAL 	:= 0;		(*Relative plate angle that should be turned at the constant speed part. Note that a negative angle leads to a movement in backward direction. Given in degree.*)
		ConstPlateSpeed :REAL 	:= 0;		(*Speed given in rps that is used for the constant speed part. The speed is seen as an absolute value (use positive speed also for backward movement).*)
		DecTime : 		UINT 	:= 5;		(*Relative time (duration) which is used to decelerate from the given speed to standstill. Given in ms.*)
		Input : 			USINT 	:= 1;		(*Selects if the first or the second input is used. Value range is {1; 2}*)
		SensorType :		USINT 	:= 1;		(*Specifies the signal source: 1 = analog input; 2 = digital input*)
		Threshold : 		REAL 	:= 0;		(*Threshold for sensor signal. Negative value is used for inverse mark polarity. Only needed for analogue sensors.*)
		MinLength : 		UINT 	:= 0;		(*Specifies the minimum length that is recognized as a mark. Given in number of samples.
										 MinLength has to be smaller than MaxLength.*)
		MaxLength :		UINT 	:= 0;		(*Specifies the maximum length that is recognized as a mark. Given in number of samples.
										 MaxLength has to be bigger than MinLength.*)
	END_STRUCT;
	DD_PatternAlignment_DDS :
	STRUCT
		AccTime : 		UINT 	:= 5;		(*Relative time (duration) which is used to accelerate from standstill to the desired speed. Given in ms.*)
		ConstPlateAngle : 	REAL 	:= 0;		(*Relative plate angle that should be turned at the constant speed part. Note that a negative angle leads to a movement in backward direction. Given in degree.*)
		ConstPlateSpeed :REAL 	:= 0;		(*Speed given in rps that is used for the constant speed part. The speed is seen as an absolute value (use positive speed also for backward movement).*)
		DecTime : 		UINT 	:= 5;		(*Relative time (duration) which is used to decelerate from the given speed to standstill. Given in ms.*)
		Input : 			USINT 	:= 1;		(*Selects if the first or the second input is used. Value range is {1; 2}*)
		Threshold : 		REAL 	:= 0;		(*Threshold for correlation of the sensor signal with the reference pattern for recognition.*)
		SubSample : 		USINT 	:= 1;		(*Subsampling factor for the sensor input. Here the subsampling rate has to be given which has been used when recording the pattern, e.g.
										 1 = 16kHz or 20 kHz (depending on the fieldbus cycle time)
										 2 = 8 or 10 kHz
										 4 = 4 or 5 kHz
										 8 = 2 or 2.5 kHz
										 16 = 1or 1.25 kHz*)
		CheckLength :	UINT 	:= 1;		(*Number of consecutive descending correlation samples after correlation maximum. Recommended value is 1.*)
	END_STRUCT;
	DD_RelativePositioning_DDS :
	STRUCT
		PlateAngle :	REAL := 0;	(*Relative angle that the rotor turns (using a P5)*)
		StartVel :		REAL := 0;	(*Velocity of the axis at the beginning of this block. The velocity has to be given as a factor between the velocity of the axis in relation to the velocity of the guide value.*)
		EndVel : 		REAL := 0;	(*Same as StartVel but at the end of the block.*)
		StartAcc :	REAL := 0;	(*Acceleration of the axis at the beginning of this block. The acceleration has to be given as a factor between the acceleration of the axis in relation to the acceleration of the guide value.*)
		EndAcc : 	REAL := 0;	(*Same as StartAcc but at the end of the block.*)
	END_STRUCT;
	DD_StaticAlignment_DDS :
	STRUCT
		StartVel :					REAL := 0;						(*Velocity of the axis at the beginning of this block. The velocity has to be given as a factor between the velocity of the axis in relation to the velocity of the guide value.*)
		EndVel : 					REAL := 0;						(*Same as StartVel but at the end of the block.*)
		StartAcc :				REAL := 0;						(*Acceleration of the axis at the beginning of this block. The acceleration has to be given as a factor between the acceleration of the axis in relation to the acceleration of the guide value.*)
		EndAcc : 				REAL := 0;						(*Same as StartAcc but at the end of the block.*)
		NodeID :					UINT := 0;						(*ID of the start of this static alignment block. This ID is needed to switch between the alternatives.
																The value 0 is only allowed (and then mandatory) if it is the first CAM block of a Labeling CAM.*)
		NumOfAlternatives: 		UINT := 0;						(*Number of alternative paths that should be created. Up to this number, the following plate angles are evaluated. Value range= [1; 8]*)
		PlateAngle: 				ARRAY [1..8] OF REAL := 8(0);		(*Relative plate angle that the rotor turns (using a P5); If there has been no switching command, the angle given at index 1 will be used per default; Given in degree.*)
		SegmentID: 				ARRAY [1..8] OF UINT := 8(0);		(*ID of the relative movement according to the angle that has been given for parameter PlateAngle[x]. The segment IDs need to be unique for this CAM.*)
	END_STRUCT;
END_TYPE
