
FUNCTION_BLOCK SAB_REF
	(*SAB_REF: Representation of the SAB for Danfoss Drive Servo with EtherCAT*)
	VAR_INPUT
		ModuleOk : BOOL; 						(*References ModuleOK from AutomationStudio*)
		NodeNumber : USINT; 					(* Node number of the device *)
		SlotName : STRING[80]; 				(*Name of the POWERLINK slot that this axis is physically connected to*)
		TPDO : ARRAY[0..8] OF UDINT; 		(* Data for IO Mapping *)
	END_VAR
	VAR_OUTPUT
		RPDO : ARRAY[0..8] OF UDINT; 		(* Data for IO Mapping *)
	END_VAR
	VAR
		intern: 						iDD_SAB_REF_INTERN_SAB; 	(* internal variable *)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadParameter4_SAB
	(*DD_ReadParameter4: FB for SAB:
	FB to asynchronously read general objects of up to 4 bytes from the object dictionary of the SAB*)
	VAR_INPUT
		Enable : 			BOOL := FALSE; 			(*Get the value of the parameter continuously while enabled*)
		Index : 			UINT := 0; 				(*Index of the object to be read*)
		Subindex : 		USINT := 0; 				(*Subindex of the object to be read*)
	END_VAR
	VAR_OUTPUT
		Valid : 			BOOL; 					(*A valid output is available at the FB*)
		Busy : 			BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		AbortCode : 		DWORD; 				(*SDO abort code in case of error*)
		Value : 			DWORD; 				(*Value of the specified parameter*)
	END_VAR
	VAR_IN_OUT
		Sab : 			SAB_REF; 				(*Reference to the SAB*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_READ_SAB; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadParameter_SAB
	(*DD_ReadParameter: FB for SAB:
	FB to asynchronously read general objects of more than 4 bytes from the object dictionary of the SAB*)
	VAR_INPUT
		Enable : 			BOOL := FALSE; 			(*Get the value of the parameter continuously while enabled*)
		Index : 			UINT := 0; 				(*Index of the object to be read*)
		Subindex : 		USINT := 0; 				(*Subindex of the object to be read*)
		pBuffer : 			UDINT := 0; 				(*Pointer to a buffer where the read data shall be placed*)
		BufferSize : 		UDINT := 0; 				(*Maximum size of the read data (size of the provided buffer given in Byte; use SIZEOF() function)*)
	END_VAR
	VAR_OUTPUT
		Valid : 			BOOL; 					(*A valid output is available at the FB*)
		Busy : 			BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo :		DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		AbortCode : 		DWORD; 				(*SDO abort code in case of error*)
		DataLength : 		UDINT; 					(*Length of the read data (in byte)*)
	END_VAR
	VAR_IN_OUT
		Sab : 			SAB_REF; 				(*Reference to the SAB*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_READ_SAB; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_WriteParameter4_SAB
	(*DD_WriteParameter4: FB for SAB:
	FB to asynchronously write general objects of up to 4 bytes to the object dictionary of the SAB*)
	VAR_INPUT
		Execute : 		BOOL := FALSE; 			(*Write the value of the parameter at rising edge*)
		Index : 			UINT := 0; 				(*Index of the object to be written*)
		Subindex : 		USINT := 0; 				(*Subindex of the object to be written*)
		Length : 			USINT := 0; 				(*Length of the data in byte that shall be written*)
		Value : 			DWORD := 0; 				(*Value of the specified parameter*)
	END_VAR
	VAR_OUTPUT
		Done : 			BOOL; 					(*A valid output is available at the FB*)
		Busy : 			BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		AbortCode : 		DWORD; 				(*SDO abort code in case of error*)
	END_VAR
	VAR_IN_OUT
		Sab :			SAB_REF; 				(*Reference to the SAB*)
	END_VAR
	VAR
		intern : 			iDD_INTERN_SDO_WRITE_SAB; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_WriteParameter_SAB
	(*DD_WriteParameter: FB for SAB:
	FB to asynchronously write general objects of more than 4 bytes to the object dictionary of the SAB*)
	VAR_INPUT
		Execute : 		BOOL := FALSE; 			(*Write the value of the parameter at rising edge*)
		Index : 			UINT := 0; 				(*Index of the object to be written*)
		Subindex : 		USINT := 0; 				(*Subindex of the object to be written*)
		Length : 			USINT := 0; 				(*Length of the data in byte that shall be written*)
		pBuffer : 			REFERENCE TO UDINT; 		(*Pointer to the buffer that contains the data that shall be written*)
	END_VAR
	VAR_OUTPUT
		Done : 			BOOL; 					(*A valid output is available at the FB*)
		Busy : 			BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS;		(*Error identification and instance identifier*)
		AbortCode : 		DWORD; 				(*SDO sbort code in case of error*)
	END_VAR
	VAR_IN_OUT
		Sab :			SAB_REF; 				(*Reference to the SAB*)
	END_VAR
	VAR
		intern :			iDD_INTERN_SDO_WRITE_SAB; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadVersion_SAB
	(*DD_ReadVersion: FB for SAB:
	FB to read the version information out f the SAB*)
	VAR_INPUT
		Execute : 			BOOL := FALSE; 			(*Read the information at rising edge*)
	END_VAR
	VAR_OUTPUT
		Done : 				BOOL; 					(*A valid set of outputs is available at the FB*)
		Busy : 				BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 				BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 			DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		SerialNumber : 		STRING[18]; 				(*Serial number of the SAB*)
		MajorVersionNo : 		UINT; 					(*Major firmware number*)
		MinorVersionNo : 		UINT; 					(*Minor firmware number*)
		BetaVersionNo : 		UINT; 					(*Beta firmware number*)
		SoftwareType : 		UINT; 					(*Loaded software type*)
	END_VAR
	VAR_IN_OUT
		Sab : 				SAB_REF; 				(*Reference to the SAB*)
	END_VAR
	VAR
		intern : iDD_INTERN_READ_VERS_SAB; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadDcLinkVoltage_SAB
	(*DD_ReadDcLinkVoltage: FB for SAB:
	This function block returns the value of the actual DC link voltage as long as "Enable" is set.*)
	VAR_INPUT
		Enable : 			BOOL := FALSE; 			(*Get the value of the parameter continuously while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : 			BOOL; 					(*A valid output is available at the FB*)
		Busy : 			BOOL;					(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		Voltage : 		INT; 					(*The value of the actual DC link voltage [V]*)
	END_VAR
	VAR_IN_OUT
		Sab : 			SAB_REF; 				(*Reference to the SAB*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadDcLinkPower_SAB
	(*DD_ReadDcLinkPower: FB for SAB:
	This function block returns the value of the actual DC link power as long as "Enable" is set.*)
	VAR_INPUT
		Enable : 			BOOL := FALSE; 				(*Get the value of the parameter continuously while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid :			BOOL; 						(*A valid output is available at the FB*)
		Busy : 			BOOL; 						(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 						(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 			(*Error identification and instance identifier*)
		Power : 			REAL; 						(*The value of the actual DC link power [Ampere]*)
	END_VAR
	VAR_IN_OUT
		Sab : 			SAB_REF; 					(*Reference to the SAB*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadPosGuideValueRef_SAB
	(*DD_ReadPosGuideValueRef: FB for SAB:
	This function block returns the value of the position guide value reference as long as "Enable" is set.*)
	VAR_INPUT
		Enable : 				BOOL := FALSE; 			(*Get the value of the parameter continuously while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : 				BOOL; 					(*A valid output is available at the FB*)
		Busy : 				BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 				BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 			DD_ERROR_DDS;		(*Error identification and instance identifier*)
		PosGuideValue : 		UDINT; 					(*The value of the position guide value reference*)
	END_VAR
	VAR_IN_OUT
		Sab : 				SAB_REF; 				(*Reference to the SAB*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadVelGuideValueRef_SAB
	(*DD_ReadVelGuideValueRef: FB for SAB:
	This function block returns the value of the velicty guide value reference as long as "Enable" is set.*)
	VAR_INPUT
		Enable : 				BOOL := FALSE; 			(*Get the value of the parameter continuously while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : 				BOOL; 					(*A valid output is available at the FB*)
		Busy : 				BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 				BOOL;					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 			DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		VelGuideValue : 		REAL; 					(*The value of the velocity guide value reference*)
	END_VAR
	VAR_IN_OUT
		Sab : 				SAB_REF; 				(*Reference to the SAB*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadSabInfo_SAB
	(*DD_ReadSabInfo: FB for SAB:
	This function block reads information concerning a SAB, like modes and certain status information.*)
	VAR_INPUT
		Enable : 						BOOL := FALSE; 			(*Get the SAB information constantly while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : 						BOOL; 					(*A valid set of outputs is available at the FB*)
		Busy : 						BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 						BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 					DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		CommunicationReady : 		BOOL; 					(*"Network" is initialized and ready for communication*)
		ReadyForPowerOn : 			BOOL; 					(*SAB is ready to be enabled (power on)*)
		PowerOn : 					BOOL;					(*If TRUE shows that the power stage is switched ON*)
		SabWarning : 				BOOL; 					(*Warning(s) on the SAB is present*)
		SabError : 					BOOL;					(*Error(s) on the SAB is present*)
	END_VAR
	VAR_IN_OUT
		Sab : 						SAB_REF; 				(*Reference to the SAB*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadSabError_SAB
	(*DD_ReadSabError: FB for SAB:
	This function block presents general SAB errors not relating to the function blocks
	(for instance SAB errors, voltage errors, communication errors).*)
	VAR_INPUT
		Enable : 			BOOL := FALSE; 			(*Get the SAB information constantly while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid : 			BOOL; 					(*A valid output is available at the FB*)
		Busy : 			BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		SabErrorID : 		WORD; 					(*The value of the SAB error*)
	END_VAR
	VAR_IN_OUT
		Sab : 			SAB_REF; 				(*Reference to the SAB*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_READ_SAB; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_ReadSabWarning_SAB
	(*DD_ReadSabWarning: FB for SAB:
	This function block presents general SAB warnings not relating to the function blocks *)
	VAR_INPUT
		Enable : 				BOOL := FALSE; 			(*Get the SAB information constantly while enabled*)
	END_VAR
	VAR_OUTPUT
		Valid :				BOOL; 					(*A valid output is available at the FB*)
		Busy : 				BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 				BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 			DD_ERROR_DDS; 		(*Error identification and instance identifier*)
		SabWarningID : 		WORD; 					(*The value of the SAB warning*)
	END_VAR
	VAR_IN_OUT
		Sab : 				SAB_REF; 				(*Reference to the SAB*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_READ_SAB; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_Power_SAB
	(*DD_Power: PLCopen FB for SAB:
	This function block controls the DC link voltage on the output lines (not the FB itself)*)
	VAR_INPUT
		Enable : 			BOOL := FALSE;				(*As long as this input is true, DC link voltage is being enabled on the output lines*)
		TimeLimit : 		TIME := t#0ms;				(*Time out after which an error is signalled, if the Status has not changed to true while Enable is true. 0 will disable the TimeLimit*)
	END_VAR
	VAR_OUTPUT
		Status : 			BOOL; 						(*Effective state of the DC link voltage on the output lines*)
		Valid : 			BOOL; 						(*If true, a valid set of outputs is available at the FB*)
		Error : 			BOOL; 						(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 			(*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Sab : 			SAB_REF;					(*Reference to the SAB*)
	END_VAR
	VAR
		intern : iDD_INTERN_POWER_DDS; (*internal variable*)
		pSabIntern : REFERENCE TO iDD_SAB_REF_INTERN_SAB; (*internal variable of SAB*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_Trace_SAB
	(*DD_Trace: FB for SAB:
	This function block is used to start a trace in the SAB with the settings given in the input variables. After
	sending the settings and starting the trace, the FB will poll the SAB until the data has been recorded
	and afterwards uploads the data automatically.*)
	VAR_INPUT
		Execute : 				BOOL := FALSE; 									(*Starts the trace functionality at rising edge and keeps on polling*)
		Abort : 					BOOL := FALSE; 									(*Abort the on-going trace. Note that new values are only evaluated on a rising edge of Execute.*)
		pTraceBuffer : 			UDINT; 											(*Reference to a buffer where the read trace data shall be placed*)
		TraceBufferSize : 			UDINT := 0; 										(*Size of the trace buffer (size of the provided buffer given in Byte; use SIZEOF() function)*)
		SamplingRate : 			DD_SAMPLING_RATE_DDS := ddFastTask_DDS; 		(*Sampling rate of the trace*)
		SampleCount : 			UINT := 4000; 										(*Number of samples to be traced per channel*)
		SubSampling : 			UINT := 1; 										(*Multiplier to adjust time difference between trace samples*)
		SignalIDs : 				ARRAY[0..7] OF UDINT :=[ 8(0)]; 						(*Holds the IDs of the signals that should be traced*)
		TriggerID : 				UDINT := 0; 										(*ID of the signal used for triggering; Use 0 to have instantaneous tracing.*)
		TriggerPoint : 				USINT := 10; 										(*Holds the information on how many pre-trigger history is traced [in percentage]*)
		TriggerLevel : 			REAL := 0.0; 										(*Level upon which will be triggered*)
		TriggerSlope : 			BOOL := TRUE; 									(*TRUE: will trigger on rising slope; FALSE: will trigger on falling slope*)
	END_VAR
	VAR_OUTPUT
		Done : 					BOOL; 											(*The trace has successfully been recorded and read from the device.*)
		Busy : 					BOOL; 											(*The FB is not finished and new output values are to be expected*)
		CommandAborted : 		BOOL; 											(*Trace has been aborted.*)
		Error : 					BOOL; 											(*Signals that an error has occurred within the FB*)
		ErrorInfo : 				DD_ERROR_DDS;								(*Error identification and instance identifier*)
		Status : 					USINT; 											(*Holds the status of the tracing:
																				 1= Trace configured and started
																				 2= Waiting for trigger
																			  	 3= Triggered
																				 4= Uploading trace data*)
		TraceLength : 			UDINT; 											(*Length of trace data that has been uploaded*)
	END_VAR
	VAR_IN_OUT
		Sab :					SAB_REF; 										(*Reference to the SAB*)
	END_VAR
	VAR
		intern :					iDD_INTERN_TRACE_SAB; 						(*internal variable*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_Reset_SAB
	(*DD_Reset: PLCopen FB for SAB:
	This function block makes the transition from the state "Error" to "Standby" by resetting all internal SAB-related errors - 
	it does not affect the output of the FB instances.*)
	VAR_INPUT
		Execute : 		BOOL := FALSE; 			(*Resets all internal SAB-related errors*)
	END_VAR
	VAR_OUTPUT
		Done : 			BOOL; 					(*A valid output is available at the FB*)
		Busy : 			BOOL; 					(*The FB is not finished and new output values are to be expected*)
		Error : 			BOOL;					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 		DD_ERROR_DDS; 		(*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Sab : 			SAB_REF; 				(*Reference to the SAB*)
	END_VAR
	VAR
		intern : iDD_INTERN_RESET_DDS; (*internal variable*)
		pSabIntern : REFERENCE TO iDD_SAB_REF_INTERN_SAB; (*internal variable of SAB*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_SimulateGuideValue_SAB
	(*DD_SimulateGuideValue: FB for SAB:
	This function block instructs the SAB to do a simulation of a guide value with the given parameters.
	The guide value ref can be read with the FBs DD_ReadPosGuideValueRef_SAB and DD_ReadVelGuideValueRef_SAB*)
	VAR_INPUT
		Enable : 				BOOL := FALSE; 			(*As long as Enable is true, the guide value is simulated by the SAB. Enable = FALSE will stop the simulation immediately!*)
		Velocity :				REAL := 0.0;				(*Velocity of the guide value. Given in rps*)
		Acceleration :			REAL := 0.0;				(*Acceleration that is used for the guide value calculation. Given in rps*)
		Deceleration :			REAL := 0.0;				(*Deceleration that is used for guide value calculation. Given in rps*)
	END_VAR
	VAR_OUTPUT
		Valid : 				BOOL; 					(*A valid output is available at the FB*)
		Busy : 				BOOL;					(*The FB is not finished and new output values are to be expected*)
		Error : 				BOOL; 					(*Signals that an error has occurred within the FB*)
		ErrorInfo : 			DD_ERROR_DDS; 		(*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Sab : 				SAB_REF; 				(*Reference to the SAB*)
	END_VAR
	VAR
		intern : iDD_INTERN_BASIC_DDS; (*internal variable*)
		pSabIntern : REFERENCE TO iDD_SAB_REF_INTERN_SAB; (*internal variable of SAB*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_UpdateFirmware_SAB
	(*DD_UpdateFirmware: FB for SAB:
	This function block is used to update the appropriate firmware to a SAB. The update of the firmware is
	only allowed in Standby state. After successful file transfer, the SAB will automatically start
	updating and after that, a power-cycle is required to go back to normal operation.*)
	VAR_INPUT
		Execute : 			BOOL := FALSE; 						(*Starts the firmware update at a rising edge*)
		FileName : 			STRING[80] := ''; 						(*Filename of the firmware file on the PLC*)
		pDevice : UDINT; 		(*Pointer to the name of the PLC device on which the file is located*)
	END_VAR
	VAR_OUTPUT
		Done : 				BOOL; 								(*The firmware update process has successfully transferred the firmware to the device*)
		Busy : 				BOOL; 								(*The FB is not finished and new output values are to be expected*)
		Error : 				BOOL; 								(*Signals that an error has occurred within the FB*)
		ErrorInfo : 			DD_ERROR_DDS; 					(*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Sab : 				SAB_REF; 							(*Reference to the SAB*)
	END_VAR
	VAR
		intern : iDD_INTERN_UPDATE_FW_DDS; (*internal variable*)
		pSabIntern : REFERENCE TO iDD_SAB_REF_INTERN_SAB; (*internal variable of SAB*)
	END_VAR
END_FUNCTION_BLOCK

FUNCTION_BLOCK DD_SetGuideValueRef_SAB
	(*DD_SetGuideValueRef: FB for SAB:
	This function block sets the position guide value reference object to the given value.
	The internally calculated offset will be stored permanently inside the SAB. 
	It is recommended that the guide value is constant during usage of this FB. 
	The value has to be given in guide value units.*)
	VAR_INPUT
		Execute : 		BOOL := FALSE;			(*Sets the position guide value reference object to the given value*)
		Position : 		UDINT := 0;				(*Value of the position guide value reference object that should be set*)
	END_VAR
	VAR_OUTPUT
		Done : 			BOOL;					(*The value has successfully been set in the SAB.*)
		Busy : 			BOOL;					(*The FB is not finished and new output values are to be expected.*)
		Error : 			BOOL;					(*Signals that an error has occurred within the FB.*)
		ErrorInfo : 		DD_ERROR_DDS;		(*Error identification and instance identifier*)
	END_VAR
	VAR_IN_OUT
		Sab : 			SAB_REF; 				(*Reference to the SAB*)
	END_VAR
	VAR
		intern : iDD_INTERN_SDO_WRITE_SAB; (*internal variable*)
	END_VAR
END_FUNCTION_BLOCK
